module advectx

#include "advect.h"

  implicit none

  ! --- interface

  private

  public :: advectxzoom, put_xedges


contains

  subroutine advectxzoom( region )

    ! set parameters for advectx 
    ! written by patrick berkvens and mike botchev, march-june 1999
    ! updated and modified by MK, dec 2002

    use dims,          only : xref, yref, zref, tref, im, jm, lm
    use dims,          only : status, nsplitsteps, n_operators, zoom2D
    use dims,          only : touch_sp, touch_np, adv_scheme, okdebug
    use dims,          only : parent, splitorderzoom, zero
#ifdef with_budgets    
    use budget_global, only : sum_advection
#endif
    use global_data  , only : mass_dat, wind_dat
    use toolbox,       only : escape_tm
#ifdef MPI 
    use mpi_const,     only : myid, lmloc, my_real, com_lev, ierr
    use mpi_const,     only : root_t, mpi_sum, ntracetloc, pe_first_tracer
    use mpi_comm,      only : barrier_t
#endif

    implicit none

    ! input/output
    integer,intent(in)                  :: region

    ! local
    real,dimension(:,:,:),  pointer     :: am

    integer                             :: js,je,ls,le,n,q
    integer                             :: imr,jmr,lmr,tref_,xref_,yref_,zref_
    real,dimension(:,:),allocatable     :: am0,am1  ! for slopes only
    logical                             :: x_encountered
    character(len=1)                    :: dir
    real                                :: sum_old, sum_new
    real                                :: sum_old_all, sum_new_all

    ! start

#ifdef MPI
    if ( ntracetloc == 0 ) return !WP! only PE's with nonzero lmloc proceed
#endif

    call put_xedges(region)   ! write BC to cildren

    tref_ = tref(region)/tref(parent(region))
    xref_ = xref(region)/xref(parent(region))
    yref_ = yref(region)/yref(parent(region))
    zref_ = zref(region)/zref(parent(region))
    imr = im(region)
    jmr = jm(region)
    lmr = lm(region)

    allocate(am0(0:jm(region)+1, 0:lmr+1))
    allocate(am1(0:jm(region)+1, 0:lmr+1))

    am => wind_dat(region)%am_t

    sum_old = sum(mass_dat(region)%rm_t(1:im(region),1:jm(region),1:lmr,1))

    ! determine the scope for advectx: 

    if ( region == 1 ) then
       yref_ = 0 
       zref_ = 0  ! to have js/je and ls/le properly computed
    end if
    q=status(region)/((nsplitsteps/2)*tref_)  
    ! find q - the place in the splitorderzoom 
    ! corresponding to the begining of the last 
    ! triple x-y-z of the parent
    x_encountered=.false.

    do n=1,n_operators    
       ! now track four following steps from q !wp! changed to n_op.. steps

       dir=splitorderzoom(region,q*(nsplitsteps/2)*tref_+n)
       select case(dir)
       case('x') 
          x_encountered=.true.
       case('y') 
          if ( .not. x_encountered) then  
             js=1                             ! y-substep is before x => 
             je=jm(region)                    ! full j-scope 
          else
             if( touch_sp(region) == 1 ) then
                js=1                             !CMK special case...touching SP
             else
                js=yref_+1                       ! x-substep is before y =>
             end if
             if( touch_np(region) == 1 ) then
                je=jm(region)                    ! CMK special case: touching NP
             else
                je=jm(region)-yref_              ! restricted j-scope
             end if
          end if
       case('z') 
          if  ( .not. x_encountered ) then
             ls=1                             ! z-substep is before x => 
             le=lmr                    ! full l-scope 
          else
             ls=zref_+1                       ! x-substep is before z =>
             le=lmr-zref_              ! restricted l-scope
          end if
          if (zoom2D) then
             ls=1; le=lmr    !WP! this is always the case
          end if
       case ('c') 
       case ('v')
       case ('s') 
       case default
          print *,'advectxzoom: strange value in splitorderzoom(',region,',',   &
               q*(nsplitsteps/2)*tref_+n,'): ',q
          call escape_tm( 'advectxzoom: error in x-advection ')
       end select

    end do  ! n=1,n_operators

    if ( ( mod(status(region),(nsplitsteps/2)*tref_) >= n_operators ) .and. &
         ( adv_scheme == 'slope' ) .and. &
         ( im(region)/xref(region) < im(1) ) ) then
       ! IF (1) more than fourth substep, (2) slope scheme, and 
       ! (3) the region is not [0;360] degrees wide THEN
       ! at this substep no coarse fluxes will be applied 

       ! for slopes only: zero out velocities via the x-edges of the region
       ! to assure that no fluxes will be applied

       if ( okdebug ) print *,'advectxzoom: zeroing out outer mass fluxes'
       am0 = am(    xref_-1,:,:)
       am(    xref_-1,:,:) = zero
       am1 = am(imr-xref_+1,:,:)
       am(imr-xref_+1,:,:) = zero

    end if

    call advectx_work(region,js,je,ls,le)

    if ( ( mod(status(region),(nsplitsteps/2)*tref_) >= n_operators ).and. &
         ( adv_scheme == 'slope' ) .and. &
         ( im(region)/xref(region) < im(1) ) ) then

       ! for slopes only: recreate velocities via the x-edges
       am(    xref_-1,:,:) = am0
       am(imr-xref_+1,:,:) = am1

    end if

    sum_new = sum(mass_dat(region)%rm_t(1:im(region),1:jm(region),1:lmr,1))

#ifdef with_budgets
#ifdef MPI
    if( myid == pe_first_tracer ) &
         sum_advection(region) = sum_advection(region) + sum_new - sum_old
    if( okdebug .and. myid == root_t ) &
         print *, 'advectxzoom: region, sum_advection ',&
         region,sum_advection(region),sum_new,sum_old
#else
    sum_advection(region) = sum_advection(region) + sum_new - sum_old
    if ( okdebug ) print *, 'advectxzoom: region, sum_advection ', &
         region,sum_advection(region),sum_new,sum_old
#endif
#endif

    deallocate(am0)
    deallocate(am1)
    nullify(am)

  end subroutine advectxzoom



  subroutine advectx_work(region,js,je,ls,le)
    !
    ! makes reduced grid pre-/postprocessing and switches between 
    ! dynamu and dynamu1
    ! written by mike botchev, march-june 1999
    !
    use dims,        only : adv_scheme, im, jm, lm
    use redgridZoom, only : grid_reduced, nred, uni2red, uni2red_mf, red2uni_em
#ifdef secmom
	  use redgridZoom, only : red2uni_em_2nd, uni2red_2nd
#endif
    use global_data, only : mass_dat, wind_dat
    use toolbox,     only : escape_tm
#ifdef MPI
    use mpi_const,   only : lmloc
#endif
    use chem_param,  only : ntracet

    implicit none

    ! input
    integer,intent(in) :: region
    integer,intent(in) :: js
    integer,intent(in) :: je
    integer,intent(in) :: ls
    integer,intent(in) :: le

    ! local
    real,dimension(:,:,:),pointer     :: m
    real,dimension(:,:,:),pointer     :: am
    real,dimension(:,:,:),allocatable :: m_uni   ! for reduced grid...
    real,dimension(:,:,:),allocatable :: am_uni  ! for reduced grid...
    real                 :: mxval
    integer,dimension(3) :: mxloc
    integer              :: imr,jmr,lmr

    ! start
	 
    if ( adv_scheme /= 'slope' .and. adv_scheme /= '2nd_m') then
       call escape_tm( 'advectx: Wrong advection scheme: adv_scheme/=slope or /=2nd_m')
    end if

    imr = im(region) ; jmr = jm(region) ; lmr = lm(region)

    allocate(m_uni(-1:im(region)+2,-1:jm(region)+2, lmr))
    allocate(am_uni(0:im(region)+1, 0:jm(region)+1, 0:lmr+1))

    am => wind_dat(region)%am_t
    m  => mass_dat(region)%m_t

    ! transform to the reduced grid:
    ! check for reduced grid in region

    if ( grid_reduced .and. (nred(region) /= 0) ) then 
       ! save non-reduced m and am in m_uni and am_uni:
       m_uni = m
       am_uni = am
       ! reduce m,rm,rxm,rym,rzm:
			if(adv_scheme=='slope')call uni2red(region)
#ifdef secmom
			if(adv_scheme=='2nd_m')call uni2red_2nd(region)
#endif
       ! reduce am:
       call uni2red_mf(region)
    end if

       if (adv_scheme=='slope') call dynamu (region,js,je,ls,le)
#ifdef secmom       
       if (adv_scheme=='2nd_m') call dynamu_2nd (region,js,je,ls,le)
#endif

    ! transform from the reduced grid:

    if ( grid_reduced .and. nred(region) /= 0 ) then
       ! advection on uniform grid:
       m_uni(1:imr,1:jmr,1:lmr) = m_uni(1:imr,1:jmr,1:lmr) + &
            am_uni(0:imr-1,1:jmr,1:lmr) - am_uni(1:imr,1:jmr,1:lmr)

       ! redistribute rm,rxm,rym,rzm by using m_uni and m
       ! recreate rm/rxm/rym/rzm by using equal masses (not m_uni) 
   	if(adv_scheme=='slope')call red2uni_em(region)
#ifdef secmom
   	if(adv_scheme=='2nd_m')call red2uni_em_2nd(region)
#endif
       ! recreate am:
       am = am_uni
    end if

    nullify(am)
    nullify(m)

    deallocate(am_uni)
    deallocate(m_uni)

  end subroutine advectx_work

  subroutine dynamu(region,js,je,ls,le)
    !-----------------------------------------------------------------------
    !
    !****   dynamu  - east-west tracer transport  - v 9.1
    !
    !   programmed by           mh      mpi HH          23-feb-1995
    !
    !   purpose
    !   -------
    !   calculate amount of tracer moved in an east-west advection
    !   substep
    !
    !   interface
    !   ---------
    !   call dynamu
    !
    !   method
    !   ------
    !   slopes scheme
    !
    !   externals
    !   ---------
    !   none
    !
    !   reference
    !   ---------       
    !   Russel and Lerner, 1979
    !-----------------------------------------------------------------------
    !   fixed bug in calculating maximum courant number
    !                          mh, Thu, Feb 24, 1994 12:49:27
    !     included code for limits of slopes to prevent negative tracer 
    !     masses                   mh, 20-jun-1994
    !     zoom version written by mike botchev, march-june 1999
    !-----------------------------------------------------------------------

    use dims,        only : okdebug, nregions, parent
    use dims,        only : im, jm, lm, xref, yref, zref, tref
    use dims,        only : zero, one, xi, nxi, nloop_max, limits, xcyc
    use redgridZoom, only : grid_reduced, nred, imred
    use global_data, only : mass_dat, wind_dat
    use zoom_tools,  only : mix_edges
    use toolbox,     only : escape_tm
#ifdef with_budgets    
    use budget_global, only : budget_flux, iflux1, iflux2, apply_budget_global
#endif
#ifdef MPI
    use mpi_const,   only : ntracetloc,myid,root_t,com_trac
    use mpi_const,   only : ierr,my_real,mpi_max,mpi_min, ntracet_ar
    use mpi_comm,    only : barrier_t
#endif
    use chem_param,  only : ntracet, ra
    implicit none

    ! input/output
    integer,intent(in) :: region
    integer,intent(in) :: js
    integer,intent(in) :: je
    integer,intent(in) :: ls
    integer,intent(in) :: le

    ! local
    real,dimension(:,:,:,:),pointer    :: rm,rxm,rym,rzm
    real,dimension(:,:,:),  pointer    :: m,am
    real,dimension(:),allocatable              :: f,pf,fy,fz
    real,dimension(:,:,:),allocatable          :: mnew
    integer i,ie,is,j,l,n, nglob, offsetn
    integer imr,jmr,lmr,tref_,xref_,yref_,zref_,ic,nloop, iloop
    real :: min_one,max_one,min_all,max_all
    real :: alpha, max_alpha, x, rmold
    real,dimension(:), allocatable :: mx
    logical                        :: cfl_ok
    integer, parameter             :: max_nloop = 50

    ! start

    if ( okdebug ) print *,'dynamu: region=',region, &
         ' js,je,ls,le=',js,je,ls,le
    if ( (region < 0) .or. (region > nregions) ) &
         call escape_tm( 'dynamu: STOP, illegal number of region !!!')

    ! prepare x-edges to allow for uniform work:
    
    call compress_xedges(region,js,je,ls,le)

    ! compute refinement factors with respect to the parent
    xref_ = xref(region)/xref(parent(region))
    yref_ = yref(region)/yref(parent(region))
    zref_ = zref(region)/zref(parent(region))
    tref_ = tref(region)/tref(parent(region))
    imr=im(region);jmr=jm(region);lmr=lm(region)

    allocate(f(0:im(region)))
    allocate(pf(0:im(region)))
    allocate(fy(0:im(region)))
    allocate(fz(0:im(region)))
    allocate(mnew(im(region),jm(region),lmr))

    am => wind_dat(region)%am_t
    m => mass_dat(region)%m_t
    rm => mass_dat(region)%rm_t
    rxm => mass_dat(region)%rxm_t
    rym => mass_dat(region)%rym_t
    rzm => mass_dat(region)%rzm_t

    ! check js,je,ls,le:
    if((js/=yref_+1)  .and.(js/=1))   stop 'dynamu: Wrong value for JS '
    if((je/=jmr-yref_).and.(je/=jmr)) stop 'dynamu: Wrong value for JE '
    if((ls/=zref_+1)  .and.(ls/=1))   stop 'dynamu: Wrong value for LS '
    if((le/=lmr-zref_).and.(le/=lmr)) stop 'dynamu: Wrong value for LE '

    ! compute is/ie -- cells is:ie,js:je:ls:le will be updated
    is = xref_; ie = imr-xref_+1
    
    if ((region)/xref(region)==im(1)) then
       ! periodic boundary condition
       is = 1; ie = imr
    end if

    ! if requested limit zonal slopes such that no negative
    ! tracer masses should occur
    if ( limits ) then
       !forall(i=is:ie,j=js:je,l=ls:le)
       do j=js,je
          ! reduced grid
          if (grid_reduced.and.nred(region).ne.0) ie = imred(j,region)
          do i=is-1,ie+1    !JdL  problems with negatives solved.....
             do l=ls,le
                rxm(i,j,l,:)=max(min(rxm(i,j,l,:),rm(i,j,l,:)),-rm(i,j,l,:))
             end do
          end do
       end do !end forall
    end if

    if ( okdebug ) then

       max_one=maxval(m(1:imr,1:jmr,1:lmr))
       min_one=minval(m(1:imr,1:jmr,1:lmr))
#ifdef MPI   
       call barrier_t
       call mpi_reduce(max_one,max_all,1,my_real,mpi_max,root_t,com_trac,ierr)
       call mpi_reduce(min_one,min_all,1,my_real,mpi_min,root_t,com_trac,ierr)
       if ( myid == root_t ) print *,'dynamu: m_min,m_max: ',min_all,max_all
       max_one=maxval(rm(1:imr,1:jmr,1:lmr,:))
       min_one=minval(rm(1:imr,1:jmr,1:lmr,:))
       call mpi_reduce(max_one,max_all,1,my_real,mpi_max,root_t,com_trac,ierr)
       call mpi_reduce(min_one,min_all,1,my_real,mpi_min,root_t,com_trac,ierr)
       if ( myid == root_t ) print *,'dynamu: rm_min,rm_max: ',min_all,max_all
#else
       print *,'dynamu: m_min,m_max: ',min_one, max_one
       max_one=maxval(rm(1:imr,1:jmr,1:lmr,:))
       min_one=minval(rm(1:imr,1:jmr,1:lmr,:))
       print *,'dynamu: rm_min,rm_max: ',min_one,max_one
#endif

    end if

    ! loop over vertical layers and latitudes
    do j=js,je
       do l=ls,le
          ! reduced grid: less points will be handled
          if (grid_reduced.and.nred(region).ne.0) ie = imred(j,region)

          ! Determine number of loops required to avoid CFLs...   
          nloop = 1   ! default run the loop one time!
          alpha = 2.0
          allocate(mx(is-1:ie+1))
          max_alpha = 0.0

          !CMK added oct2003: nloop limit..
          do while ( abs(alpha) >= one .and. nloop < max_nloop )
             mx(is-1:ie+1) = m(is-1:ie+1,j,l)   ! copy mass to temp array
             if(xcyc(region)==1) then
                mx(0) = mx(ie)
                mx(ie+1) = mx(1)
             end if

             xloop: do iloop = 1, nloop
                cfl_ok = .true.
                do i=is-1,ie   
                   if (am(i,j,l)>=zero)  then
                      alpha=am(i,j,l)/mx(i)
                   else
                      alpha=am(i,j,l)/mx(i+1)
                   end if
                   if((abs(alpha)>=one)) then                   !PB
                      cfl_ok = .false.
                      exit xloop
                   end if
                   max_alpha = max(max_alpha, abs(alpha))
                end do
                ! update mass
                ! note: only is:ie updated, is-1, ie+1 are BC
                mx(is:ie) = mx(is:ie) + am(is-1:ie-1,j,l) - am(is:ie,j,l)
                if ( xcyc(region) == 1 ) then   !except for periodic BC...
                   mx(0) = mx(ie)
                   mx(ie+1) = mx(1)
                end if
             end do  xloop
             if(.not.cfl_ok) then 
                ! reduce mass flux
                am(is-1:ie,j,l) = am(is-1:ie,j,l)*nloop/(nloop+1)
                if ( okdebug ) &
                     print *, 'dynamu: ',j,l, 'nloop increased to', &
                     nloop + 1, alpha
                nloop = nloop + 1
                max_alpha = 0.0
                if (nloop == max_nloop) call escape_tm('dynuamu: nloop too high')
             end if
          end do !while alpha>1
          deallocate(mx)
          nloop_max(region,1) = max(nloop_max(region,1),nloop)
          xi(region,1) = max(xi(region,1),max_alpha)          
          do iloop = 1,nloop   ! CFL loop
             ! calculate new air mass distribution
             mnew(is:ie,j,l)=m(is:ie,j,l) + am(is-1:ie-1,j,l)-am(is:ie,j,l)
             
             if (im(region)/xref(region)==im(1)) then
                m  (0    ,j,l)   = m  (ie,j,l) !periodic BCs...
                m  (ie+1,j,l)    = m  (1,  j,l)
             end if

             ! loop over tracers
#ifdef MPI   
             do n=1,ntracetloc
                offsetn=sum(ntracet_ar(0:myid-1))
                nglob = n + offsetn             

#else
             do n=1,ntracet
                nglob = n
#endif
                if (im(region)/xref(region)==im(1)) then
                   rm (0,    j,l,n) = rm (ie,j,l,n)   !periodic BCs
                   rxm(0,    j,l,n) = rxm(ie,j,l,n)
                   rym(0,    j,l,n) = rym(ie,j,l,n)
                   rzm(0,    j,l,n) = rzm(ie,j,l,n)
                   rm (ie+1,j,l,n) = rm (1,  j,l,n)
                   rxm(ie+1,j,l,n) = rxm(1,  j,l,n)
                   rym(ie+1,j,l,n) = rym(1,  j,l,n)
                   rzm(ie+1,j,l,n) = rzm(1,  j,l,n)
                end if

                ! calculate fluxes for rm,rxm,rym,rzm
                do i=is-1,ie
                   if (am(i,j,l)>=zero)  then
                      alpha=am(i,j,l)/m(i,j,l)
                      f(i)=alpha*(rm(i,j,l,n)+(one-alpha)*rxm(i,j,l,n))
                      pf(i)=am(i,j,l)*(alpha*alpha*rxm(i,j,l,n) - 3.*f(i))
                      fy(i)=alpha*rym(i,j,l,n)
                      fz(i)=alpha*rzm(i,j,l,n)
                   else
                      alpha=am(i,j,l)/m(i+1,j,l)
                      f(i)=alpha*(rm(i+1,j,l,n)-(one+alpha)*rxm(i+1,j,l,n))
                      pf(i)=am(i,j,l)*(alpha*alpha*rxm(i+1,j,l,n) - 3.*f(i))
                      fy(i)=alpha*rym(i+1,j,l,n)
                      fz(i)=alpha*rzm(i+1,j,l,n)
                   end if
                   !xi(region,1)=max(xi(region,1),abs(alpha))
                   if(abs(alpha)>=one) then
                      if (okdebug) then
                         print *,'dynamu: CFL violation: alpha,m=',alpha, &
                              m(i,j,l),'  in region,i,j,l: ',region,i,j,l
                      end if
                      nxi(region,1)=nxi(region,1)+1
                   end if
                   
                end do
                
                if (im(region)/xref(region)==im(1)) then
                   ! redefine left fluxes using periodicity:
                   ! reduced grid
                   if (grid_reduced.and.nred(region).ne.0) imr = imred(j,region)
                   f(0)  = f (imr)
                   pf(0) = pf(imr)
                   fy(0) = fy(imr)
                   fz(0) = fz(imr)
                end if
                
#ifdef with_budgets    
                ! 
                ! add up flux budget!
                ! note that this fails if grid is reduced (ie check)
                !
                if ( apply_budget_global ) then 
                  if ( (iflux1(region)-1 >= is) .and. (iflux1(region)-1 <= ie) ) then 
                     budget_flux(region)%flux_x1(j,l,nglob) =  budget_flux(region)%flux_x1(j,l,nglob) &
                        + f(iflux1(region)-1)*1e3/ra(nglob)   ! moles
                  endif
                  if ( (iflux2(region)-1 >= is) .and. (iflux2(region)-1 <= ie) ) then 
                     budget_flux(region)%flux_x2(j,l,nglob) =  budget_flux(region)%flux_x2(j,l,nglob) &
                        + f(iflux2(region)-1)*1e3/ra(nglob)
                  endif
                end if
#endif
                !
                !calculate new tracer mass, and tracer mass slopes
                !
                do i=is,ie
                   rm(i,j,l,n)=rm(i,j,l,n)+(f(i-1)-f(i))
                   rxm(i,j,l,n)=rxm(i,j,l,n)+(pf(i-1)-pf(i)       &
                        -(am(i-1,j,l)-am(i,j,l))*rxm(i,j,l,n)     &
                        +3.*((am(i-1,j,l)+am(i,j,l))*rm(i,j,l,n)  &
                        -(f(i-1)+f(i))*m(i,j,l)))/mnew(i,j,l)
                   !CMK: apply limits again: might be in nloop!
                   if ( limits ) rxm(i,j,l,n) = &
                        max(min(rxm(i,j,l,n),rm(i,j,l,n)),-rm(i,j,l,n))
                   rym(i,j,l,n)=rym(i,j,l,n)+(fy(i-1)-fy(i))
                   rzm(i,j,l,n)=rzm(i,j,l,n)+(fz(i-1)-fz(i))
                end do !forall
             end do  ! end of n-loop
             ! store new air mass in m array
             m(is:ie,j,l)=mnew(is:ie,j,l)

          end do  ! iloop

          ! restore 'old' am
          am(is-1:ie,j,l) = am(is-1:ie,j,l)*nloop

       end do
    end do    ! end of l-loop/j-loop

    if (im(region)/xref(region)==im(1)) then
       ! periodic boundary condition
       do j=js,je
          ! reduced grid
          if (grid_reduced.and.nred(region).ne.0) imr = imred(j,region)
          rm (0,    j,:,:) = rm (imr,j,:,:)
          rxm(0,    j,:,:) = rxm(imr,j,:,:)
          rym(0,    j,:,:) = rym(imr,j,:,:)
          rzm(0,    j,:,:) = rzm(imr,j,:,:)
          rm (imr+1,j,:,:) = rm (1,  j,:,:)
          rxm(imr+1,j,:,:) = rxm(1,  j,:,:)
          rym(imr+1,j,:,:) = rym(1,  j,:,:)
          rzm(imr+1,j,:,:) = rzm(1,  j,:,:)
          m  (0    ,j,:)   = m  (imr,j,:)
          m  (imr+1,j,:)   = m  (1,  j,:)
       end do
    end if

    deallocate(f)
    deallocate(pf)
    deallocate(fy)
    deallocate(fz)
    deallocate(mnew)

    nullify(am)
    nullify(m)
    nullify(rm)
    nullify(rxm)
    nullify(rym)
    nullify(rzm)

    call uncompress_xedges(region,js,je,ls,le)
    call mix_edges(region)

  end subroutine dynamu

  subroutine dynamu_2nd(region,js,je,ls,le)
    !-----------------------------------------------------------------------
    !
    !****   dynamu_2nd  - east-west tracer transport  - v 9.1
    !
    !
    !	purpose
    !	-------
    !	calculate amount of tracer moved in an east-west advection
    !	substep
    !
    !	interface
    !	---------
    !	call dynamu
    !
    !	method
    !	------
    !	slopes scheme
    !
    !	externals
    !	---------
    !	none
    !
    !	reference
    !	---------	
    !	Prather, 1986
    !-----------------------------------------------------------------------
    !	fixed bug in calculating maximum courant number
    !		               mh, Thu, Feb 24, 1994 12:49:27
    !     included code for limits of slopes to prevent negative tracer 
    !     masses                   mh, 20-jun-1994
    !     zoom version written by mike botchev, march-june 1999
	  !     2nd moments : Bram Bregman, Jul 2004.
    !     updated to include budgets and homogenise with new base : Jason Williams, May 2006
    !-----------------------------------------------------------------------

    use dims,        only : okdebug, nregions, parent
    use dims,        only : im, jm, lm, xref, yref, zref, tref
    use dims,        only : zero, one, xi, nxi, nloop_max, limits, xcyc, limits_extra
    use redgridZoom, only : grid_reduced, nred, imred
    use global_data, only : mass_dat, wind_dat
    use zoom_tools,  only : mix_edges
    use toolbox,     only : escape_tm
	  use partools,		 only : root_t
#ifdef with_budgets
    use budget_global, only : budget_flux, iflux1, iflux2, apply_budget_global 
#endif   
#ifdef MPI
    use mpi_const,   only : ntracetloc,com_trac, ntracet_ar
    use mpi_const,   only : ierr,myid,my_real,mpi_max,mpi_min,pe_first_tracer
    use mpi_comm,    only : barrier_t
#endif
    use chem_param,  only : ntracet, ra
    implicit none

    ! input/output
    integer,intent(in) :: region
    integer,intent(in) :: js
    integer,intent(in) :: je
    integer,intent(in) :: ls
    integer,intent(in) :: le

    ! local
    real,dimension(:,:,:,:),pointer    :: rm,rxm,rym,rzm,rxxm,rxym,rxzm,ryym,ryzm,rzzm
    real,dimension(:,:,:),  pointer    :: m,am
	  real,dimension(:),allocatable      :: f,pf,fy,fz,f0,fx,fxx,fyy,fzz,fxy,fxz,fyz,fm,&
											   		alpha_i,alph,alf,alf1,alf1q
    real,dimension(:,:,:),allocatable  :: mnew,amold, mold
	  integer 									         :: i,ie,is,j,l,n, iloop, ip1, nloop, offsetn
    integer 									         :: imr,jmr,lmr,tref_,xref_,yref_,zref_,ic,nglob
    real 										           :: min_one,max_one,min_all,max_all
    real 										           :: alpha,x,rmold, max_alpha,temptm, alfq, blf, blf1
    real,dimension(:), allocatable 		 :: mx
    logical                        		 :: cfl_ok
	  integer,dimension(3)				       :: mnloc,mxloc
	  real											         :: slpmax, s1new, s1max, s2new
    integer,parameter                  :: max_nloop = 50

    ! start

    if ( okdebug ) print *,'dynamu: region=',region, &
         ' js,je,ls,le=',js,je,ls,le
    if ( (region < 0) .or. (region > nregions) ) &
         call escape_tm( 'dynamu: STOP, illegal number of region !!!')

    ! prepare x-edges to allow for uniform work:
    
    call compress_xedges(region,js,je,ls,le)

    ! compute refinement factors with respect to the parent
    xref_ = xref(region)/xref(parent(region))
    yref_ = yref(region)/yref(parent(region))
    zref_ = zref(region)/zref(parent(region))
    tref_ = tref(region)/tref(parent(region))
    imr=im(region);jmr=jm(region);lmr=lm(region)

    allocate(f(0:im(region)))
    allocate(f0(0:im(region)))
    allocate(fm(0:im(region)))
    allocate(pf(0:im(region)))
    allocate(fx(0:im(region)))
    allocate(fy(0:im(region)))
    allocate(fz(0:im(region)))
    allocate(fxx(0:im(region)))
    allocate(fxy(0:im(region)))
    allocate(fxz(0:im(region)))
    allocate(fyy(0:im(region)))
    allocate(fyz(0:im(region)))
    allocate(fzz(0:im(region)))
    allocate(alph(0:im(region)+1))
    allocate(alpha_i(0:im(region)+1))
    allocate(alf(0:im(region)))
    allocate(alf1(0:im(region)))
    allocate(alf1q(0:im(region)))
    allocate(mnew(im(region),jm(region),lm(region)))
    allocate(amold(0:im(region)+1,0:jm(region)+1,0:lm(region)))
    allocate(mold(-1:im(region)+2,-1:jm(region)+2,lm(region)))

    am => wind_dat(region)%am_t
    m => mass_dat(region)%m_t
    rm => mass_dat(region)%rm_t
    rxm => mass_dat(region)%rxm_t
    rym => mass_dat(region)%rym_t
    rzm => mass_dat(region)%rzm_t
    rxxm => mass_dat(region)%rxxm_t
    rxym => mass_dat(region)%rxym_t
    rxzm => mass_dat(region)%rxzm_t
    ryym => mass_dat(region)%ryym_t
    ryzm => mass_dat(region)%ryzm_t
    rzzm => mass_dat(region)%rzzm_t

    ! check js,je,ls,le:
    if((js/=yref_+1)  .and.(js/=1))   stop 'dynamu_2nd: Wrong value for JS '
    if((je/=jmr-yref_).and.(je/=jmr)) stop 'dynamu_2nd: Wrong value for JE '
    if((ls/=zref_+1)  .and.(ls/=1))   stop 'dynamu_2nd: Wrong value for LS '
    if((le/=lmr-zref_).and.(le/=lmr)) stop 'dynamu_2nd: Wrong value for LE '

    ! compute is/ie -- cells is:ie,js:je:ls:le will be updated
    is = xref_; ie = imr-xref_+1
    
    if ((region)/xref(region)==im(1)) then
       ! periodic boundary condition
       is = 1; ie = imr
    end if

    ! if requested limit zonal slopes such that no negative
    ! tracer masses should occur
 
if (limits) then
#ifdef MPI
	do n = 1,ntracetloc
#else
  do n = 1,ntracet
#endif  
		do l = ls,le
			do j = js,je
      		do i=is-1,ie+1    !otherwise negative
            slpmax=0.0
						if (rm(i,j,l,n).gt.0.) slpmax=rm(i,j,l,n)
						 s1max=1.5*slpmax
             s1new=amin1(+s1max,amax1(-s1max,rxm(i,j,l,n)))
             s2new=amin1((slpmax+slpmax-0.3334*abs(s1new)),amax1(abs(s1new)-slpmax,rxxm(i,j,l,n)))
						 rxym(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxym(i,j,l,n)))
						 rxzm(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxzm(i,j,l,n)))
						 rxm(i,j,l,n) =s1new
						 rxxm(i,j,l,n)=s2new
         	enddo
			enddo
		enddo
	enddo
endif

    if ( okdebug ) then
       max_one=maxval(m(1:imr,1:jmr,1:lmr))
       min_one=minval(m(1:imr,1:jmr,1:lmr))
#ifdef MPI   
       call barrier_t
       call mpi_reduce(max_one,max_all,1,my_real,mpi_max,root_t,com_trac,ierr)
       call mpi_reduce(min_one,min_all,1,my_real,mpi_min,root_t,com_trac,ierr)
        if ( myid == root_t ) then
		 	   print *,'dynamu: m_min,m_max: ',min_all,max_all
       	  do n = 1, ntracetloc																    
		    	  max_one=maxval(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr))		    
          	min_one=minval(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr))		    
          	mxloc=maxloc(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr)) 		    
          	mnloc=minloc(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr)) 		    
          	print *, '(p)dynamu:Min+loc-Max+loc',n, min_one,mnloc,max_one,mxloc  
		   	  enddo	
		     endif																				    
       call mpi_reduce(max_one,max_all,1,my_real,mpi_max,root_t,com_trac,ierr)
       call mpi_reduce(min_one,min_all,1,my_real,mpi_min,root_t,com_trac,ierr)
#else
       do n = 1, ntracet
		 	  max_one=maxval(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr))
       	min_one=minval(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr))
       	mxloc=maxloc(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr))
       	mnloc=minloc(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr))
       	print *, 'dynamu:Min+loc-Max+loc',n, min_one,mnloc,max_one,mxloc
		 enddo
#endif
    end if ! okdebug

    ! loop over vertical layers and latitudes
    do j=js,je
       do l=ls,le
          ! reduced grid: less points will be handled
          if (grid_reduced.and.nred(region).ne.0) ie = imred(j,region)

          ! Determine number of loops required to avoid CFLs...   
          nloop = 1   ! default run the loop one time!
          alpha = 2.0
          allocate(mx(is-1:ie+1))
          max_alpha = 0.0
          !CMK added oct2003: nloop limit..
          do while ( abs(alpha) >= one .and. nloop < max_nloop )
             mx(is-1:ie+1) = m(is-1:ie+1,j,l)   ! copy mass to temp array
             if(xcyc(region)==1) then
                mx(0) = mx(ie)
                mx(ie+1) = mx(1)
             end if

             xloop: do iloop = 1, nloop
                cfl_ok = .true.
                do i=is-1,ie   
                   if (am(i,j,l)>=zero)  then
                      alpha=am(i,j,l)/mx(i)
                   else
                      alpha=am(i,j,l)/mx(i+1)
                   end if
                   if((abs(alpha)>=one)) then                   !PB
                      cfl_ok = .false.
                      exit xloop
                   end if
                   max_alpha = max(max_alpha, abs(alpha))
                end do
                ! update mass
                ! note: only is:ie updated, is-1, ie+1 are BC
                mx(is:ie) = mx(is:ie) + am(is-1:ie-1,j,l) - am(is:ie,j,l)
                if ( xcyc(region) == 1 ) then   !except for periodic BC...
                   mx(0) = mx(ie)
                   mx(ie+1) = mx(1)
                end if
             end do  xloop
             if(.not.cfl_ok) then 
                ! reduce mass flux
                am(is-1:ie,j,l) = am(is-1:ie,j,l)*nloop/(nloop+1)
                if ( okdebug ) &
                     print *, 'dynamu: ',j,l, 'nloop increased to', &
                     nloop + 1, alpha
                nloop = nloop + 1
                max_alpha = 0.0
                if(nloop == max_nloop) call escape_tm('dynuamu: nloop too high')
                if ( nloop > max_nloop) &
                  call escape_tm(' too many iteration in advectx !! ' )
             end if
          end do !while alpha>1
          deallocate(mx)
          nloop_max(region,1) = max(nloop_max(region,1),nloop) 
          xi(region,1) = max(xi(region,1),max_alpha)
          do iloop = 1,nloop   ! CFL loop
             ! calculate new air mass distribution
             mnew(is:ie,j,l)=m(is:ie,j,l) + am(is-1:ie-1,j,l)-am(is:ie,j,l)
             
             if (im(region)/xref(region)==im(1)) then
                m  (0    ,j,l)   = m  (ie,j,l) !periodic BCs...
                m  (ie+1,j,l)    = m  (1,  j,l)
             end if

 ! set mass for innter tracer loop
					mold(:,j,l) = m(:,j,l)
					 
             ! loop over tracers
#ifdef MPI   
             do n=1,ntracetloc
                offsetn=sum(ntracet_ar(0:myid-1))
                nglob = n + offsetn
#else
             do n=1,ntracet
                nglob = n
#endif

! reset mass for each tracer
						m(:,j,l) = mold(:,j,l)
								
                if (im(region)/xref(region)==im(1)) then
                   rm (0,j,l,n) = rm (ie,j,l,n)   !periodic BCs
                   rxm(0,j,l,n) = rxm(ie,j,l,n)
                   rym(0,j,l,n) = rym(ie,j,l,n)
                   rzm(0,j,l,n) = rzm(ie,j,l,n)
                   rxxm(0,j,l,n) = rxxm(ie,j,l,n)
                   rxym(0,j,l,n) = rxym(ie,j,l,n)
                   rxzm(0,j,l,n) = rxzm(ie,j,l,n)
                   ryym(0,j,l,n) = ryym(ie,j,l,n)
                   ryzm(0,j,l,n) = ryzm(ie,j,l,n)
                   rzzm(0,j,l,n) = rzzm(ie,j,l,n)
                   rm (ie+1,j,l,n) = rm (1,j,l,n)
                   rxm(ie+1,j,l,n) = rxm(1,j,l,n)
                   rym(ie+1,j,l,n) = rym(1,j,l,n)
                   rzm(ie+1,j,l,n) = rzm(1,j,l,n)
        				   rxxm(ie+1,j,l,n) = rxxm(1,j,l,n)
        				   rxym(ie+1,j,l,n) = rxym(1,j,l,n)
        				   rxzm(ie+1,j,l,n) = rxzm(1,j,l,n)
        				   ryym(ie+1,j,l,n) = ryym(1,j,l,n)
        				   ryzm(ie+1,j,l,n) = ryzm(1,j,l,n)
        				   rzzm(ie+1,j,l,n) = rzzm(1,j,l,n)
                 end if

    ! if requested limit zonal slopes such that no negative
    ! tracer masses should occur: second 
            if (limits) then
      			 do i=is-1,ie+1    !otherwise negative
         				slpmax=0.0
						    if (rm(i,j,l,n).gt.0.) slpmax=rm(i,j,l,n)
						    s1max=1.5*slpmax
                s1new=amin1(+s1max,amax1(-s1max,rxm(i,j,l,n)))
                s2new=amin1((slpmax+slpmax-0.3334*abs(s1new)),amax1(abs(s1new)-slpmax,rxxm(i,j,l,n)))
						    rxym(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxym(i,j,l,n)))
						    rxzm(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxzm(i,j,l,n)))
						    rxm(i,j,l,n) =s1new
						    rxxm(i,j,l,n)=s2new
         		 enddo
            endif

                ! calculate fluxes for rm,rxm,rym,rzm
! calculate flux and moments between boxes (i,j,l,n)<-->(ip1,j,l,n)
                i=ie
                do ip1=is,ie

! flux from (i) to (i+1) when am.gt.0
! create temporary moments/masses for partial boxes in transit

                   if (am(i,j,l)>=0.0) then
                      fm(i)=am(i,j,l)
                      alf(i)=fm(i)/m(i,j,l)
                      alfq=alf(i)*alf(i)
                      alf1(i)=1.0-alf(i)
                      alf1q(i)=alf1(i)*alf1(i)
                      f0(i)=alf(i)*(rm(i,j,l,n)+alf1(i)* &
							        (rxm(i,j,l,n)+(alf1(i)-alf(i))*rxxm(i,j,l,n)))
                      fx(i)=alfq*(rxm(i,j,l,n)+3.*alf1(i)*rxxm(i,j,l,n))
                      fxx(i)=alf(i)*alfq*rxxm(i,j,l,n)
                      fy(i)=alf(i)*(rym(i,j,l,n)+alf1(i)*rxym(i,j,l,n))
                      fz(i)=alf(i)*(rzm(i,j,l,n)+alf1(i)*rxzm(i,j,l,n))
                      fxy(i)=alfq*rxym(i,j,l,n)
                      fxz(i)=alfq*rxzm(i,j,l,n)
                      fyy(i)=alf(i)*ryym(i,j,l,n)
                      fzz(i)=alf(i)*rzzm(i,j,l,n)
                      fyz(i)=alf(i)*ryzm(i,j,l,n)
                   else
                      fm(i)=-am(i,j,l)
                      alf(i)=fm(i)/m(ip1,j,l)
                      alfq=alf(i)*alf(i)
                      alf1(i)=1.0-alf(i)
                      alf1q(i)=alf1(i)*alf1(i)
                      f0(i)=alf(i)*(rm(ip1,j,l,n)-alf1(i)* &
							        (rxm(ip1,j,l,n)-(alf1(i)-alf(i))*rxxm(ip1,j,l,n)))
                      fx(i)=alfq*(rxm(ip1,j,l,n)-3.*alf1(i)*rxxm(ip1,j,l,n))
                      fxx(i)=alf(i)*alfq*rxxm(ip1,j,l,n)
                      fy(i)=alf(i)*(rym(ip1,j,l,n)-alf1(i)*rxym(ip1,j,l,n))
                      fz(i)=alf(i)*(rzm(ip1,j,l,n)-alf1(i)*rxzm(ip1,j,l,n))
                      fxy(i)=alfq*rxym(ip1,j,l,n)
                      fxz(i)=alfq*rxzm(ip1,j,l,n)
                      fyy(i)=alf(i)*ryym(ip1,j,l,n)
                      fzz(i)=alf(i)*rzzm(ip1,j,l,n)
                      fyz(i)=alf(i)*ryzm(ip1,j,l,n)
                   endif
                   xi(region,1)=max(xi(region,1),alf(i))
                   i = ip1
                enddo
                  
! readjust moments remaining in the box; 
! only after calculating all the moments !!!!!!mk
                i=ie
                do ip1=is,ie
                   if (am(i,j,l)>=0.0) then
                    	 m(i,j,l) = m(i,j,l)-fm(i)
                      rm(i,j,l,n)=rm(i,j,l,n)-f0(i)
                      rxm(i,j,l,n)=alf1q(i)*(rxm(i,j,l,n)-3.*alf(i)*rxxm(i,j,l,n))
                      rxxm(i,j,l,n)=alf1(i)*alf1q(i)*rxxm(i,j,l,n)
                      rym(i,j,l,n)=rym(i,j,l,n)-fy(i)
                      ryym(i,j,l,n)=ryym(i,j,l,n)-fyy(i)
                      rzm(i,j,l,n)=rzm(i,j,l,n)-fz(i)
                      rzzm(i,j,l,n)=rzzm(i,j,l,n)-fzz(i)
                      rxym(i,j,l,n)=alf1q(i)*rxym(i,j,l,n)
                      rxzm(i,j,l,n)=alf1q(i)*rxzm(i,j,l,n)
                      ryzm(i,j,l,n)=ryzm(i,j,l,n)-fyz(i)
                   endif
                   i = ip1
                   if(limits_extra)then !BB negatives might occur due to iteration, apply limits again!
                      slpmax=0.
						        if (rm(i,j,l,n).gt.0.) slpmax=rm(i,j,l,n)
						         s1max=1.5*slpmax
                     s1new=amin1(+s1max,amax1(-s1max,rxm(i,j,l,n)))
                     s2new=amin1((slpmax+slpmax-0.3334*abs(s1new)),amax1(abs(s1new)-slpmax,rxxm(i,j,l,n)))
						         rxym(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxym(i,j,l,n)))
						         rxzm(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxzm(i,j,l,n)))
						         rxm(i,j,l,n) =s1new
						         rxxm(i,j,l,n)=s2new
                   endif
                enddo
                i=ie
                do ip1=is,ie
                   if (am(i,j,l).lt.0.0) then
                      
! flux from (i+1) to (i) when am.lt.0 (i.e. take left side of box ip1)
! readjust moments remaining in the box
                      
                    	m(ip1,j,l) = m(ip1,j,l)-fm(i)
                      rm(ip1,j,l,n)=rm(ip1,j,l,n)-f0(i)
                      rxm(ip1,j,l,n)=alf1q(i)*(rxm(ip1,j,l,n)+3.*alf(i)* rxxm(ip1,j,l,n))
                      rxxm(ip1,j,l,n)=alf1(i)*alf1q(i)*rxxm(ip1,j,l,n)
                      rym(ip1,j,l,n)=rym(ip1,j,l,n)-fy(i)
                      ryym(ip1,j,l,n)=ryym(ip1,j,l,n)-fyy(i)
                      rzm(ip1,j,l,n)=rzm(ip1,j,l,n)-fz(i)
                      rzzm(ip1,j,l,n)=rzzm(ip1,j,l,n)-fzz(i)
                      rxym(ip1,j,l,n)=alf1q(i)*rxym(ip1,j,l,n)
                      rxzm(ip1,j,l,n)=alf1q(i)*rxzm(ip1,j,l,n)
                      ryzm(ip1,j,l,n)=ryzm(ip1,j,l,n)-fyz(i)
                   endif
                   i = ip1
                   if(limits_extra)then !BB negatives might occur due to iteration, apply limits again!
                     slpmax=0.
						         if (rm(i,j,l,n).gt.0.) slpmax=rm(i,j,l,n)
						         s1max=1.5*slpmax
                     s1new=amin1(+s1max,amax1(-s1max,rxm(i,j,l,n)))
                     s2new=amin1((slpmax+slpmax-0.3334*abs(s1new)),amax1(abs(s1new)-slpmax,rxxm(i,j,l,n)))
						         rxym(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxym(i,j,l,n)))
						         rxzm(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxzm(i,j,l,n)))
						         rxm(i,j,l,n) =s1new
						         rxxm(i,j,l,n)=s2new
                   endif
                enddo
                
! put the temporary moments (fi) into appropriate neighboring boxes
                
                i=ie
                do ip1=is,ie
                   
! flux from (i) to (i+1) when am.gt.0
                   
                   if (am(i,j,l).ge.0.0) then
                      m(ip1,j,l) = m(ip1,j,l) + fm(i)
                      blf=fm(i)/m(ip1,j,l)
                      blf1=1.0-blf
                      temptm=blf*rm(ip1,j,l,n)-blf1*f0(i)
                      rm(ip1,j,l,n)=rm(ip1,j,l,n)+f0(i)
                      rxxm(ip1,j,l,n)=blf*blf*fxx(i)+blf1*blf1*rxxm(ip1,j,l,n)     &
                           +5.*(blf*blf1*(rxm(ip1,j,l,n)-fx(i))-(blf1-blf)*temptm)
                      rxm(ip1,j,l,n)=blf*fx(i)+blf1*rxm(ip1,j,l,n)+3.*temptm
                      rxym(ip1,j,l,n)=blf*fxy(i)+blf1*rxym(ip1,j,l,n)+     &
                           3.*(-blf1*fy(i)+blf*rym(ip1,j,l,n))
                      rxzm(ip1,j,l,n)=blf*fxz(i)+blf1*rxzm(ip1,j,l,n)+     &
                           3.*(-blf1*fz(i)+blf*rzm(ip1,j,l,n))
                      rym(ip1,j,l,n)=rym(ip1,j,l,n)+fy(i)
                      ryym(ip1,j,l,n)=ryym(ip1,j,l,n)+fyy(i)
                      rzm(ip1,j,l,n)=rzm(ip1,j,l,n)+fz(i)
                      rzzm(ip1,j,l,n)=rzzm(ip1,j,l,n)+fzz(i)
                      ryzm(ip1,j,l,n)=ryzm(ip1,j,l,n)+fyz(i)
                   endif
                   i=ip1
                   if(limits_extra)then !BB negatives might occur due to iteration, apply limits again!
                     slpmax=0.
						         if (rm(i,j,l,n).gt.0.) slpmax=rm(i,j,l,n)
						         s1max=1.5*slpmax
                     s1new=amin1(+s1max,amax1(-s1max,rxm(i,j,l,n)))
                     s2new=amin1((slpmax+slpmax-0.3334*abs(s1new)),amax1(abs(s1new)-slpmax,rxxm(i,j,l,n)))
						         rxym(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxym(i,j,l,n)))   
						         rxzm(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxzm(i,j,l,n)))   
						         rxm(i,j,l,n) =s1new												     
						         rxxm(i,j,l,n)=s2new												     
                   endif
                enddo
                
!#ifdef with_budgets    
                ! 
                ! add up flux budget!
                ! note that this fails if grid is reduced (ie check)
                !
                ! JEW: global budget calculation needs to be adapted for second moments
                
!                if ( apply_budget_global ) then 
!                  if ( (iflux1(region)-1 >= is) .and. (iflux1(region)-1 <= ie) ) then 
!                     budget_flux(region)%flux_x1(j,l,nglob) =  budget_flux(region)%flux_x1(j,l,nglob) &
!                        + f(iflux1(region)-1)*1e3/ra(nglob)   ! moles
!                  endif
!                  if ( (iflux2(region)-1 >= is) .and. (iflux2(region)-1 <= ie) ) then 
!                     budget_flux(region)%flux_x2(j,l,nglob) =  budget_flux(region)%flux_x2(j,l,nglob) &
!                        + f(iflux2(region)-1)*1e3/ra(nglob)
!                  endif
!                end if
!#endif                
                i=ie   !extra loop allows for vectorisation
                do ip1=is,ie
                   if (am(i,j,l).lt.0.0) then
                      
! flux from (i+1) to (i) when am.lt.0
                      
                   	 m(i,j,l) = m(i,j,l) + fm(i)
                      blf=fm(i)/m(i,j,l)
                      blf1=1.0-blf
                      temptm=-blf*rm(i,j,l,n)+blf1*f0(i)
                      rm(i,j,l,n)=rm(i,j,l,n)+f0(i)
                      rxxm(i,j,l,n)=blf*blf*fxx(i)+blf1*blf1*rxxm(i,j,l,n)+     &
                           5.*(blf*blf1*(-rxm(i,j,l,n)+fx(i))+(blf1-blf)*temptm)
                      rxm(i,j,l,n)=blf*fx(i)+blf1*rxm(i,j,l,n)+3.*temptm
                      rxym(i,j,l,n)=blf*fxy(i)+blf1*rxym(i,j,l,n)+     &
                           3.*(blf1*fy(i)-blf*rym(i,j,l,n))
                      rxzm(i,j,l,n)=blf*fxz(i)+blf1*rxzm(i,j,l,n)+     &
                           3.*(blf1*fz(i)-blf*rzm(i,j,l,n))
                      rym(i,j,l,n)=rym(i,j,l,n)+fy(i)
                      ryym(i,j,l,n)=ryym(i,j,l,n)+fyy(i)
                      rzm(i,j,l,n)=rzm(i,j,l,n)+fz(i)
                      rzzm(i,j,l,n)=rzzm(i,j,l,n)+fzz(i)
                      ryzm(i,j,l,n)=ryzm(i,j,l,n)+fyz(i)
                   endif
                   i=ip1
                   if(limits_extra)then !BB negatives might occur due to iteration, apply limits again!
                      slpmax=0.0
						        if (rm(i,j,l,n).gt.0.) slpmax=rm(i,j,l,n)
						         s1max=1.5*slpmax
                     s1new=amin1(+s1max,amax1(-s1max,rxm(i,j,l,n)))
                     s2new=amin1((slpmax+slpmax-0.3334*abs(s1new)),amax1(abs(s1new)-slpmax,rxxm(i,j,l,n)))
						         rxym(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxym(i,j,l,n)))
						         rxzm(i,j,l,n)=amin1(slpmax,amax1(-slpmax,rxzm(i,j,l,n)))
						         rxm(i,j,l,n) =s1new
						         rxxm(i,j,l,n)=s2new
                    endif
                enddo
		enddo ! ntrace

! store new air mass in m array    
   		m(is:ie,j,l) = mnew(is:ie,j,l)

   enddo  ! iloop
   ! restore 'old' am
   am(is-1:ie,j,l) = am(is-1:ie,j,l)*nloop

       end do
    end do    ! end of l-loop/j-loop

    if (im(region)/xref(region)==im(1)) then
       ! periodic boundary condition
       do j=js,je
          ! reduced grid
          if (grid_reduced.and.nred(region).ne.0) imr = imred(j,region)
          rm (0,    j,:,:) = rm (imr,j,:,:)
          rxm(0,    j,:,:) = rxm(imr,j,:,:)
          rym(0,    j,:,:) = rym(imr,j,:,:)
          rzm(0,    j,:,:) = rzm(imr,j,:,:)
          rxxm(0,    j,:,:) = rxxm(imr,j,:,:)
          rxym(0,    j,:,:) = rxym(imr,j,:,:)
          rxzm(0,    j,:,:) = rxzm(imr,j,:,:)
          ryym(0,    j,:,:) = ryym(imr,j,:,:)
          ryzm(0,    j,:,:) = ryzm(imr,j,:,:)
          rzzm(0,    j,:,:) = rzzm(imr,j,:,:)
          rm (imr+1,j,:,:) = rm (1,  j,:,:)
          rxm(imr+1,j,:,:) = rxm(1,  j,:,:)
          rym(imr+1,j,:,:) = rym(1,  j,:,:)
          rzm(imr+1,j,:,:) = rzm(1,  j,:,:)
     		  rxxm(imr+1,j,:,:) = rxxm(1,  j,:,:)
      	  rxym(imr+1,j,:,:) = rxym(1,  j,:,:)
      	  rxzm(imr+1,j,:,:) = rxzm(1,  j,:,:)
      	  ryym(imr+1,j,:,:) = ryym(1,  j,:,:)
      	  ryzm(imr+1,j,:,:) = ryzm(1,  j,:,:)
      	  rzzm(imr+1,j,:,:) = rzzm(1,  j,:,:)
          m  (0    ,j,:)   = m  (imr,j,:)
          m  (imr+1,j,:)   = m  (1,  j,:)
       end do
    end if

deallocate(f)
deallocate(fm)
deallocate(f0)
deallocate(pf)
deallocate(fx)
deallocate(fy)
deallocate(fz)
deallocate(fxx)
deallocate(fxy)
deallocate(fxz)
deallocate(fyy)
deallocate(fyz)
deallocate(fzz)
deallocate(alph)
deallocate(alpha_i)
deallocate(alf)
deallocate(alf1)
deallocate(alf1q)
deallocate(mnew)
deallocate(mold)
deallocate(amold)

nullify(am)
nullify(m)
nullify(rm)
nullify(rxm)
nullify(rym)
nullify(rzm)
nullify(rxxm)
nullify(rxym)
nullify(rxzm)
nullify(ryym)
nullify(ryzm)
nullify(rzzm)

#ifdef MPI
    if ( okdebug .and. myid == pe_first_tracer ) then
     do n = 1, ntracetloc
#else
    if ( okdebug ) then
     do n = 1, ntracet
#endif
		 	  max_one=maxval(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr))
       	min_one=minval(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr))
       	mxloc=maxloc(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr))
       	mnloc=minloc(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr))
       	print *, 'final:dynamu:Min+loc-Max+loc',n, min_one,mnloc,max_one,mxloc
		 enddo
   end if
    call uncompress_xedges(region,js,je,ls,le)
    call mix_edges(region)
 
  end subroutine dynamu_2nd



  subroutine compress_xedges(region,js,je,ls,le)
    !
    ! this is for slope only: condense data at the x-edges of the zoom region
    ! to allow for uniform work in dynamu/dynamu1
    ! written by mike botchev, march-june 1999
    !
    use dims,        only : xref, parent, im, okdebug
    use global_data, only : mass_dat
#ifdef MPI
    use mpi_const,   only : ntracetloc
#endif
    use chem_param,  only : ntracet
    implicit none

    ! input
    integer,intent(in) :: region
    integer,intent(in) :: js
    integer,intent(in) :: je
    integer,intent(in) :: ls
    integer,intent(in) :: le

    ! local
    real,dimension(:,:,:,:),pointer    :: rm,rxm,rym,rzm
    real,dimension(:,:,:),  pointer    :: m
    integer :: imr,j,l,n,xref_

    ! start

    m => mass_dat(region)%m_t
    rm => mass_dat(region)%rm_t
    rxm => mass_dat(region)%rxm_t
    rym => mass_dat(region)%rym_t
    rzm => mass_dat(region)%rzm_t
    
    xref_ = xref(region)/xref(parent(region))
    imr = im(region)
    if ( (xref_ == 1 ) .or. ( im(region)/xref(region) == im(1) ) ) then
       if ( okdebug ) print *,"compress_xedges:", &
            " no refinement or periodic bc's, nothing to do"
       return
    end if
    if ( okdebug ) then
       print *,'compress_xedges: region=',region,' imr=',imr
       print *,'compress_xedges:        cells to be updated: ', &
            xref_-1,xref_,imr-xref_+1,imr-xref_+2
    end if

    !forall(j=js:je,l=ls:le)
    do j=js,je
       do l=ls,le
          !sum_m = sum(m(1:xref_,j,l),1);  m (xref_,  j,l) = sum_m
          m (xref_,  j,l) = sum(m(1:xref_,j,l))
          m (xref_-1,j,l) = m (0,j,l) 
          m (imr-xref_+1,j,l) = sum(m(imr-xref_+1:imr,j,l))
          m (imr-xref_+2,j,l) = m (imr+1,j,l) 
          
          !forall(n=1:ntracet)
#ifdef MPI   
          do n=1,ntracetloc
#else
          do n=1,ntracet
#endif
             rm (xref_,  j,l,n) = sum(rm(1:xref_,j,l,n))
             rm (xref_-1,j,l,n) = rm (0,j,l,n) 
             rxm(xref_-1,j,l,n) = rxm(0,j,l,n)
             rym(xref_-1,j,l,n) = rym(0,j,l,n) 
             rzm(xref_-1,j,l,n) = rzm(0,j,l,n)
             
             rm (imr-xref_+1,j,l,n) = sum(rm(imr-xref_+1:imr,j,l,n))
             rm (imr-xref_+2,j,l,n) = rm (imr+1,j,l,n) 
             rxm(imr-xref_+2,j,l,n) = rxm(imr+1,j,l,n)
             rym(imr-xref_+2,j,l,n) = rym(imr+1,j,l,n) 
             rzm(imr-xref_+2,j,l,n) = rzm(imr+1,j,l,n)
          end do !forall
       end do
    end do !forall

    nullify(m)
    nullify(rm)
    nullify(rxm)
    nullify(rym)
    nullify(rzm)

  end subroutine compress_xedges



  subroutine uncompress_xedges(region,js,je,ls,le)
    !
    ! distribute data at the x-edges of the zoom region
    ! over the whole interface cell
    ! written by mike botchev, march-june 1999
    !
    use dims,        only : xref, parent, im, okdebug
    use global_data, only : mass_dat
#ifdef MPI
    use mpi_const,   only : ntracetloc
#endif
    use chem_param,  only : ntracet

    implicit none

    ! input/output
    integer,intent(in) :: region
    integer,intent(in) :: js
    integer,intent(in) :: je
    integer,intent(in) :: ls
    integer,intent(in) :: le

    ! local
    real,dimension(:,:,:,:),pointer    :: rm,rxm,rym,rzm
    real,dimension(:,:,:),  pointer    :: m
    integer :: imr,j,l,n,xref_
    real    :: m_edge,rm_edge

    ! start

    m => mass_dat(region)%m_t
    rm => mass_dat(region)%rm_t
    rxm => mass_dat(region)%rxm_t
    rym => mass_dat(region)%rym_t
    rzm => mass_dat(region)%rzm_t
    
    xref_ = xref(region)/xref(parent(region))
    imr = im(region)

    if ( (xref_==1) .or. (im(region)/xref(region)==im(1)) ) then
       if ( okdebug ) print *,"uncompress_xedges:", &
            " no refinement or periodic bc's, nothing to do"
       return
    end if
    if ( okdebug ) then
       print *,'uncompress_xedges: region=',region,' imr=',imr
       print *,'uncompress_xedges: cells to be updated: ',1,':', &
            xref_,' ',imr-xref_+1,':',imr
    end if

    ! forall(j=js:je,l=ls:le)
    do j=js,je
       do l=ls,le
          m_edge = m (xref_,j,l) 
          m (1:xref_,j,l) = m_edge/xref_
          
          m_edge = m (imr-xref_+1,j,l) 
          m (imr-xref_+1:imr,j,l) = m_edge/xref_ 
          !
          !   !forall(n=1:ntracet)
#ifdef MPI   
          do n=1,ntracetloc
#else
          do n=1,ntracet
#endif
             rm_edge = rm (xref_,j,l,n) 
             rm (1:xref_,j,l,n) = rm_edge/xref_
             !      
             !*      rxm(1:xref_-1,j,l,n) = rxm(xref_,j,l,n) 
             !*      rym(1:xref_-1,j,l,n) = rym(xref_,j,l,n) 
             !*      rzm(1:xref_-1,j,l,n) = rzm(xref_,j,l,n) 
             rxm(1:xref_,j,l,n) = rxm(xref_,j,l,n)/xref_ 
             rym(1:xref_,j,l,n) = rym(xref_,j,l,n)/xref_ 
             rzm(1:xref_,j,l,n) = rzm(xref_,j,l,n)/xref_ 
             !      
             rm_edge = rm (imr-xref_+1,j,l,n) 
             rm (imr-xref_+1:imr,j,l,n) = rm_edge/xref_ 
             !
             !*      rxm(imr-xref_+2:imr,j,l,n) = rxm(imr-xref_+1,j,l,n)
             !*      rym(imr-xref_+2:imr,j,l,n) = rym(imr-xref_+1,j,l,n)
             !*      rzm(imr-xref_+2:imr,j,l,n) = rzm(imr-xref_+1,j,l,n)
             rxm(imr-xref_+1:imr,j,l,n) = rxm(imr-xref_+1,j,l,n)/xref_
             rym(imr-xref_+1:imr,j,l,n) = rym(imr-xref_+1,j,l,n)/xref_
             rzm(imr-xref_+1:imr,j,l,n) = rzm(imr-xref_+1,j,l,n)/xref_
          end do
       end do !forall
    end do !forall

    nullify(m)
    nullify(rm)
    nullify(rxm)
    nullify(rym)
    nullify(rzm)
    
  end subroutine uncompress_xedges



  subroutine put_xedges(region)
    !
    ! passes values along the x-boundaries to all the children 
    ! written by mike botchev, march-june 1999
    ! structure implemented by MK, dec 2002
    !
    use dims,          only : im, jm, lm, xref, yref, zref
    use dims,          only : ibeg, jbeg, lbeg, iend, nregions
    use dims,          only : children, okdebug
    use global_data,   only : mass_dat
#ifdef with_budgets    
    use budget_global, only : sum_advection
    use budget_fine,   only : nzon, nzon_v, budadvx
#endif    
    use chem_param,    only : ra, ntracet
#ifdef MPI
    use mpi_const
#endif
    use tracer_data,      only : par_check_domain

    implicit none

    ! in/out
    integer,intent(in)   :: region

    ! local
    real,dimension(:,:,:,:),pointer     :: rm,rxm,rym,rzm,rmc,rxmc,rymc,rzmc
    real,dimension(:,:,:),  pointer     :: m,mc
    integer            :: child,ichild,j,jp,l,lp,jmc,lmc,n,i
    integer            :: xref_,yref_,zref_
    real               :: yzref,xyzref
    real               :: sum_old,sum_new,sum_old_all,sum_new_all
    integer            :: nzne, nzne_v,nznem
    real,dimension(:,:),allocatable :: toc0,toc1,tocm,tocm1
    integer :: communicator,root_id,lglob,nglob,lmr,nt

    ! start

#ifdef MPI
    which_par=previous_par(region)

    !WP! make sure parents are on same domain
    call par_check_domain(region,'c',which_par)

    if ( which_par == 'tracer' .and. ntracetloc == 0 ) return 
    if ( which_par == 'levels' .and. lmloc == 0 ) return
#endif

    ichild = 0
    do while(ichild<children(region,0))
       ichild = ichild + 1
       child = children(region,ichild)
       xref_ = xref(child)/xref(region)
       yref_ = yref(child)/yref(region)
       zref_ = zref(child)/zref(region)

       if (im(child)/xref(child)==im(1)) then
          ! this child is [0,360] degrees wide - skip it since 
          ! periodic boundary conditions will be used
          if (okdebug)  & 
               print *,'put_xedges: child ',child,' skipped'
          cycle 
       endif

#ifdef MPI   
       if ( which_par == 'tracer' ) then
#endif

          m => mass_dat(region)%m_t
          rm => mass_dat(region)%rm_t
          rxm => mass_dat(region)%rxm_t
          rym => mass_dat(region)%rym_t
          rzm => mass_dat(region)%rzm_t

          mc => mass_dat(child)%m_t
          rmc => mass_dat(child)%rm_t
          rxmc => mass_dat(child)%rxm_t
          rymc => mass_dat(child)%rym_t
          rzmc => mass_dat(child)%rzm_t

          lmr = lm(region)
#ifdef MPI     
          nt=ntracetloc
          communicator=com_trac  !WP! assign com_trac as communicator
          root_id=root_t
#else
          nt=ntracet
#endif

#ifdef MPI   

       else if ( which_par == 'levels' ) then

          m => mass_dat(region)%m_k
          rm => mass_dat(region)%rm_k
          rxm => mass_dat(region)%rxm_k
          rym => mass_dat(region)%rym_k
          rzm => mass_dat(region)%rzm_k

          mc => mass_dat(child)%m_k
          rmc => mass_dat(child)%rm_k
          rxmc => mass_dat(child)%rxm_k
          rymc => mass_dat(child)%rym_k
          rzmc => mass_dat(child)%rzm_k

          lmr = lmloc
          nt=ntracet
          communicator=com_lev  !WP! assign com_lev as communicator
          root_id=root_k
       end if

#endif

       ! loop through the cells of x-walls of the child
       jmc = jm(child) 
       lmc = lmr
       yzref = 1./(yref_*zref_)
       xyzref = 1./(xref_*yref_*zref_)
       allocate(toc0(jmc,lmc))
       allocate(toc1(jmc,lmc))
       allocate(tocm(jmc,lmc))
       allocate(tocm1(jmc,lmc))
       do l=1,lmc
          lp = lbeg(child) + (l-1)/zref_
          do j=1,jmc
             jp = jbeg(child) + (j-1)/yref_
             toc0(j,l) = m(ibeg(child)-1,jp,lp)*yzref
             toc1(j,l) = m(ibeg(child)  ,jp,lp)*xyzref
             tocm(j,l) = m(iend(child)  ,jp,lp)*xyzref
             tocm1(j,l) = m(iend(child)+1,jp,lp)*yzref
          end do
       end do
       !     copy info to the child ....
       mc(          0,1:jmc,1:lmc) = toc0
       mc(im(child)+1,1:jmc,1:lmc) = tocm1
       do i=1,xref_
          mc(i            , 1:jmc , 1:lmc ) = toc1
          mc(im(child)+1-i, 1:jmc , 1:lmc ) = tocm
       end do
       nullify(mc)

       sum_old = 0.0
       sum_old_all = 0.0
       sum_new = 0.0
       sum_new_all = 0.0

       do n=1,nt
          do l=1,lmc
             lp = lbeg(child) + (l-1)/zref_
             do j=1,jmc
                jp = jbeg(child) + (j-1)/yref_
                toc0(j,l) = rm(ibeg(child)-1,jp,lp,n)*yzref
                toc1(j,l) = rm(ibeg(child)  ,jp,lp,n)*xyzref
                tocm(j,l) = rm(iend(child)  ,jp,lp,n)*xyzref
                tocm1(j,l) =rm(iend(child)+1,jp,lp,n)*yzref
             end do
          end do

          rmc(          0,1:jmc,1:lmc,n) = toc0
          rmc(im(child)+1,1:jmc,1:lmc,n) = tocm1

          ! calculate the mass that is associated with the 'implicit' 
          !advection that is done here....

          nglob=n

#ifdef MPI   
          if ( myid > 0 .and. which_par == 'tracer' ) &
               nglob=sum(ntracet_ar(0:myid-1))+n
#endif

          do l=1,lmc
             lglob=l
#ifdef MPI      
             if(myid>0.and.which_par=='levels') lglob=sum(lmar(0:myid-1))+l
#endif
#ifdef with_budgets
             nzne_v = nzon_v(lglob)
#endif             
             do j=1,jmc
                if (n == 1) sum_new = sum_new +  xref_*(toc1(j,l) + tocm(j,l))
                do i=1,xref_
#ifdef with_budgets                
                   nzne = nzon(i,j)
                   nznem = nzon(im(child)+1-i,j)
                   if (n == 1) sum_old = sum_old +  &
                        rmc(i,j,l,1) + rmc(im(child)-i+1,j,l,1)
                   if ( child == nregions ) then
                      budadvx(nzne,nzne_v,nglob) = budadvx(nzne,nzne_v,nglob) + &
                           (toc1(j,l) - rmc(i            ,j,l,n))/ra(nglob)*1e3
                      budadvx(nznem,nzne_v,nglob) = budadvx(nznem,nzne_v,nglob) +&
                           (tocm(j,l) - rmc(im(child)+1-i,j,l,n))/ra(nglob)*1e3
                   endif
#endif                   
                   rmc(i             ,j,l,n) = toc1(j,l)
                   rmc(im(child)+1-i ,j,l,n) = tocm(j,l)
                end do
             end do
          end do

       end do

#ifdef MPI    
       !WP! only on one when in tracer  
       if ( which_par == 'tracer' .and. myid /= pe_first_tracer ) sum_new=0.0
       !WP! only on one when in tracer  
       if ( which_par == 'tracer' .and. myid /= pe_first_tracer ) sum_old=0.0
#endif

       !WP! When in levels, each sum_new and sum_old carries the full sum 
       !WP! of each level for tracer one.
       !WP! When in tracer,sum_new and sum_old is zero on all PE's 
       !WP! except PE_first_tracer
       
#ifdef with_budgets           

#ifdef MPI    
       !add all
       call mpi_allreduce(sum_new,sum_new_all,1,my_real,mpi_sum,communicator,ierr)
       call mpi_allreduce(sum_old,sum_old_all,1,my_real,mpi_sum,communicator,ierr)
#else
       sum_new_all = sum_new
       sum_old_all = sum_old
#endif
       sum_advection(child) = sum_advection(child) + sum_new_all - sum_old_all

#ifdef MPI    
       if ( okdebug .and. myid == root_id ) &
            print *, 'put_xedges: region, sum_advection ', &
            child,sum_advection(child)
#else
       if ( okdebug ) print *, 'put_xedges: region, sum_advection ', &
            child,sum_advection(child)
#endif

#endif

       nullify(rmc)

       do n=1,nt
          do l=1,lmc
             lp = lbeg(child) + (l-1)/zref_
             do j=1,jmc
                jp = jbeg(child) + (j-1)/yref_
                toc0(j,l) = rxm(ibeg(child)-1,jp,lp,n)*yzref
                toc1(j,l) = rxm(ibeg(child)  ,jp,lp,n)*yzref   !note the yzref...
                tocm(j,l) = rxm(iend(child)  ,jp,lp,n)*yzref
                tocm1(j,l) =rxm(iend(child)+1,jp,lp,n)*yzref
             end do
          end do

          rxmc(          0,1:jmc,1:lmc,n) = toc0
          rxmc(im(child)+1,1:jmc,1:lmc,n) = tocm1
          do i=1,xref_
             rxmc(i            , 1:jmc , 1:lmc ,n) = toc1
             rxmc(im(child)+1-i, 1:jmc , 1:lmc ,n) = tocm
          end do

       end do

       nullify(rxmc)

       do n=1,nt
          do l=1,lmc
             lp = lbeg(child) + (l-1)/zref_
             do j=1,jmc
                jp = jbeg(child) + (j-1)/yref_
                toc0(j,l) = rym(ibeg(child)-1,jp,lp,n)*yzref
                toc1(j,l) = rym(ibeg(child)  ,jp,lp,n)*yzref   !note the yzref...
                tocm(j,l) = rym(iend(child)  ,jp,lp,n)*yzref
                tocm1(j,l) =rym(iend(child)+1,jp,lp,n)*yzref
             end do
          end do
          rymc(          0,1:jmc,1:lmc,n) = toc0
          rymc(im(child)+1,1:jmc,1:lmc,n) = tocm1
          do i=1,xref_
             rymc(i            , 1:jmc , 1:lmc ,n) = toc1
             rymc(im(child)+1-i, 1:jmc , 1:lmc ,n) = tocm
          end do
       end do
       nullify(rymc)

       do n=1,nt
          do l=1,lmc
             lp = lbeg(child) + (l-1)/zref_
             do j=1,jmc
                jp = jbeg(child) + (j-1)/yref_
                toc0(j,l) = rzm(ibeg(child)-1,jp,lp,n)*yzref
                toc1(j,l) = rzm(ibeg(child)  ,jp,lp,n)*yzref   !note the yzref...
                tocm(j,l) = rzm(iend(child)  ,jp,lp,n)*yzref
                tocm1(j,l) =rzm(iend(child)+1,jp,lp,n)*yzref
             end do
          end do

          rzmc(          0,1:jmc,1:lmc,n) = toc0
          rzmc(im(child)+1,1:jmc,1:lmc,n) = tocm1
          do i=1,xref_
             rzmc(i            , 1:jmc , 1:lmc ,n) = toc1
             rzmc(im(child)+1-i, 1:jmc , 1:lmc ,n) = tocm
          end do

       end do
       nullify(rzmc)
       deallocate(toc0)
       deallocate(toc1)
       deallocate(tocm)
       deallocate(tocm1)

       nullify(rm)
       nullify(rxm)
       nullify(rym)
       nullify(rzm)
       nullify(m)
    end do


  end subroutine put_xedges



end module advectx
