#! /bin/csh -ef
# --- init -----------------------------

# set program name and location: 
if ( $0 =~ /* ) then
  set script = $0
else
  set script = $PWD/$0
endif
set scriptdir = ${script:h}
set prog = ${script:t}


# --- help -----------------------------

if ( ${#argv} < 1 ) then
cat << EOF

NAME
  $prog - run TM5 model
  
USAGE
  $prog <rcfile>
  
RCFILE

  See the template 'TM5/rcfiles/tm5.rc' .
  
EOF
exit(0)
endif

# --- arguments ------------------------

set rcfile      = $1
set time_start      = $2
set time_final      = $3
set compdir = ${scriptdir}/..

# --- binaries ----------
set basename = /usr/bin/basename
if ( ! -x $basename ) then
    set basename = /bin/basename
endif
if ( ! -x $basename ) then
    echo "${prog} - ERROR!  Cannot find basename executable in /usr/bin or /bin"
    exit 1
endif

# --- begin -----------------------------------------------------------
set iserror = 'F'


echo "$prog - Read settings from ${rcfile} ..."
set rundir   = `$scriptdir/go_readrc $rcfile 'rundir'`
set outputfreq   = `$scriptdir/go_readrc $rcfile 'outputfreq'`
set nlag   = `$scriptdir/go_readrc $rcfile 'nlag'`
set regtype   = `$scriptdir/go_readrc $rcfile 'regtype'`

set with_mpi = `$scriptdir/go_readrc $rcfile 'with.mpi'`
set npes     = `$scriptdir/go_readrc $rcfile 'npes'`
if ( "${with_mpi}" != "T" ) set npes = 1
set inputdir = `$scriptdir/go_readrc $rcfile 'inputdir'`
set istart = `$scriptdir/go_readrc $rundir/tm5_runtime.rc 'istart'`
set sitesrc   = `$scriptdir/go_readrc $rcfile 'sites.rc'`

# ----------------
# check settings of rc file versus those in source code used
# ----------------
cd $compdir/src
if($with_mpi == 'T') then
   # check the NPES setting in mpi_const:
   set npesf = `grep -i 'npes =' mpi_const.F90 | awk '{print $5}'`
   if ($npes != $npesf) then
    echo "${prog} - ERROR!  npes ($npes) in rc file ($rcfile) should be equal to npes ($npesf) in mpi_const.F90"
    set iserror = 'T'
   endif
   set ntracet = `grep -i 'ntracet =' chem_param.F90 | awk '{print $6}'`
   @ tt = $ntracet % $npes  
   if($tt != 0 ) then
    echo "${prog} - ERROR!  ntracet ($ntracet) in rc file ($rcfile) should be integer multiple of npes ($npes)"
    set iserror = 'T'
   endif
endif

# check that start.state is a bare filename--no directory.
set start_state = `$scriptdir/go_readrc $rcfile 'start.state'`
if ( `${basename} $start_state` != $start_state) then
    echo "${prog} - ERROR!  start.state in $rcfile apparently has a directory specification."
    echo "${prog}   This is deprecated in favor of a bare filename, e.g. 'start.state: savestate.hdf'"
    echo "${prog}   Value in file: $start_state"
    set iserror = 'T'
endif

# check the nlag setting in enkf_constants:
set nlagf = `sed 's/\\!.*//g' enkf_constants.F90 | grep -i 'nlag[      ]*=' | awk -F\: '{print $3}' | awk -F= '{print $2}' | tr -d "'" `
# anatomy:  filter all comments                   get assignment only        get part after ::       get part after =       strip single quotes
if ($nlagf != $nlag) then
    echo "${prog} - ERROR!  nlag ($nlag) in rc file ($rcfile) should be equal to nlag ($nlagf) in enkf_constants.F90"
    set iserror = 'T'
endif

# check the timestep setting in enkf_constants:
set outputfreqf = `sed 's/\\!.*//g' enkf_constants.F90 | grep -i 'timestep[      ]*=' | awk -F\: '{print $3}' | awk -F= '{print $2}' | tr -d "'" `
# anatomy:        filter all comments                   get assignment only            get part after ::       get part after =       strip single quotes
if ($outputfreqf != $outputfreq) then
    echo "${prog} - ERROR!  outputfreq ($outputfreq) in rc file ($rcfile) should be equal to timestep ($outputfreqf) in enkf_constants.F90"
    set iserror = 'T'
endif

# check the regtype setting in enkf_constants:
set regtypef = `sed 's/\\!.*//g' enkf_constants.F90 | grep -i 'regtype[      ]*=' | awk -F\: '{print $3}' | awk -F= '{print $2}' | tr -d "'" `
# anatomy:     filter all comments                   get assignment only           get part after ::       get part after =       strip single quotes
if ($regtypef != $regtype) then
    echo "${prog} - ERROR!  regtype ($regtype) in rc file ($rcfile) should be equal to regtype ($regtypef) in enkf_constants.F90"
    set iserror = 'T'
endif

if ( $iserror == 'T' ) then
    echo "${prog} - Please fix error(s) above.  Exiting prematurely."
    exit 1
endif

# link input
#

cd $rundir

echo "$prog - link input files ..."

set transcomdir   = `$scriptdir/go_readrc $rcfile 'transcomdir'`
set dir1x1   = `$scriptdir/go_readrc $rcfile 'dir1x1'`
#set deltapco2_prefix   = `$scriptdir/go_readrc $rcfile 'deltapco2.prefix'`
#set bio_covariance =  `$scriptdir/go_readrc $rcfile 'bio.covariance'`
#set ocn_covariance =  `$scriptdir/go_readrc $rcfile 'ocn.covariance'`
set regtype =  `$scriptdir/go_readrc $rcfile 'regtype'`
    
# general input
/bin/cp $inputdir/station.list .
/bin/cp $inputdir/RedGrid .
/bin/cp $inputdir/fintwb.data  .
# 1x1 files needed
#/bin/ln -sf $dir1x1/co2_init_new.hdf co2_init.hdf
/bin/ln -sf $dir1x1/co2_init_opt.hdf co2_init.hdf
/bin/ln -sf $dir1x1/landfraction.hdf landfraction.hdf
/bin/ln -sf $dir1x1/distances.hdf .
/bin/ln -sf $dir1x1/transcom_${regtype}.hdf .
#/bin/ln -sf $dir1x1/${bio_covariance} .
#/bin/ln -sf $dir1x1/${ocn_covariance} .
# flux input from TC Continuous
#/bin/ln -sf $transcomdir/transcom_sib_hflux*.hdf .
#/bin/ln -sf $transcomdir/transcom_sib_dflux*.hdf .
#/bin/ln -sf $transcomdir/transcom_casa_dflux*.hdf .
/bin/ln -sf $dir1x1/transcom_ff95.hdf .
#/bin/ln -sf $transcomdir/transcom_taka02_flux*.hdf .
#/bin/ln -sf $dir1x1/${deltapco2_prefix}* .
/bin/ln -sf $dir1x1/bioparams* .
/bin/ln -sf $dir1x1/biofireparams* .
/bin/ln -sf $dir1x1/BBguido* .
# fossil fuel inputs
/bin/ln -sf $dir1x1/ff_input/co2ff.edgar_cdiac.seasnam.1x1.hdf* .
#flux input from old TC
#ln -sf $dir1x1/transcom_ff95.hdf transcom_ff95.hdf
#ln -sf $dir1x1/transcom_ocean.hdf transcom_ocean.hdf
#ln -sf $dir1x1/transcom_nep.hdf transcom_nep.hdf

# site selection

set blacklist = (xxx)

set idate = `echo ${time_start} | /usr/bin/cut -c 1-8`
set idate_final = `${scriptdir}/date_add ${idate} $outputfreq`

if (-f obs_final.list) rm -f obs_final.list
if (-f obs_forecast_preBL.list) rm -f obs_forecast_preBL.list

set obsdir =  `$scriptdir/go_readrc $sitesrc 'obs.dir'`
echo $obsdir

# first make list with obs for current step
#

# read the list of site categories to be used for the obs_final.list file
set n_obs_final = `$scriptdir/go_readrc $sitesrc 'n.obs.final'`
@ icomp = 0
set components = ()
echo "$prog - obs_final.list is constructed from $n_obs_final components:"
while ( $icomp < $n_obs_final) 
    @ icomp = $icomp + 1
    set component = `$scriptdir/go_readrc $sitesrc "obs.final.$icomp"`
    set components = ($components $component)
    echo "  [$icomp] " $component
end

while ($idate < $idate_final )

    foreach component ( $components )

	set obscompfile = ${obsdir}/${component}$idate

	if (-e ${obscompfile}) then
	    cat ${obscompfile} >> obs_final.list
	endif

    end # foreach
    set idate = `${scriptdir}/date_add ${idate} 1`
end # while

# length of loop depends on whether we just started the filter or not
set nstep = ${nlag}
if ( $istart == 6 ) set nstep = 1
@ int2 =  ($nstep - 1) * $outputfreq
set idates = `echo ${time_start} | /usr/bin/cut -c 1-8`
set idate = `${scriptdir}/date_add ${idates} $int2`
@ int2 =  $nlag * $outputfreq
set idate_final = `${scriptdir}/date_add ${idates} $int2`

echo $idate
echo $idate_final

# read the list of site categories to be used for the obs_forecast.list file
set n_obs_forecast = `$scriptdir/go_readrc $sitesrc 'n.obs.forecast'`
@ icomp = 0
set components = ()
echo "$prog - obs_forecast.list is constructed from $n_obs_forecast components:"
while ( $icomp < $n_obs_forecast) 
    @ icomp = $icomp + 1
    set component = `$scriptdir/go_readrc $sitesrc "obs.forecast.$icomp"`
    set components = ($components $component)
    echo "  [$icomp] " $component
end

# now make list with obs forecasted (for filter optimization)
while ($idate < $idate_final )
    foreach component ( $components )

	    set obscompfile = ${obsdir}/${component}$idate
            echo $idate $obscompfile
	    if (-e ${obscompfile}) then
		cat ${obscompfile} >> obs_forecast_preBL.list
	    endif
    end # foreach
    set idate = `${scriptdir}/date_add ${idate} 1`
end # while

rm -f blacklist.txt
cp $scriptdir/select_obs.awk .
foreach stat ($blacklist)
   awk -f select_obs.awk f1=$stat obs_forecast_preBL.list > obs.list
   cp obs.list obs_forecast_preBL.list
end
mv obs_forecast_preBL.list obs_forecast.list
set nfinal = `wc -l obs_final.list`
echo "$prog - obs_final.list has $nfinal observations."
set nfcast = `wc -l obs_forecast.list`
echo "$prog - obs_forecast.list has $nfcast observations."

# leave the obs lists behind in archive-friendly format
set idate = `echo ${time_start} | /usr/bin/cut -c 1-8`
cp obs_forecast.list obs_forecast.list.${idate}
cp obs_final.list obs_final.list.${idate}
cp pseudo.list pseudo.list.${idate}

# In forward mode, we need hdf files with parameters.  These are
# assumed to be called "xac_01" and located in a file called
# "savestate.hdf".  There should be a different savestate.hdf for each
# week, located in a subdirectory named using the 8-digit idate.  The
# rcfile key "fwd.parms.dir" gives the location of the parent of these subdirs.
# For instance, if fwd.parms.dur = /p73/co2/andy/enso/output, then
# we expect hdf files of the form /p73/co2/andy/enso/output/20010101/savestate.hdf.
#
# Parameters are read by the routine "ReadParams" in enkf_common,
# which is called by "start" in initexit.F90.
#
set fwd_parms_dir = `$scriptdir/go_readrc $rcfile 'fwd.parms.dir' missing`
if ( "$fwd_parms_dir" != "missing" ) then
    set source_file = ${fwd_parms_dir}/${idate}/savestate.hdf
    if (! -e ${source_file} ) then
	echo "${prog} - Cannot find ${source_file}"
	echo "${prog} - Is fwd.parms.dir value of ${fwd_parms_dir} correct in rc file?"
	echo "${prog} - Exiting."
	exit 1
    endif
    ln -sf ${source_file} fwd_parms.hdf
endif

# --- end --------------------------------------------------------------
