!### macros #####################################################
!
#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!#################################################################

module chemloss
  !BOP
  ! !DESCRIPTION:
  !-----------------------------------------------------------------------
  !       purpose
  !       -------
  !       perform emissions needed for TM5 CBM4 version
  !
  !       interface
  !       ---------
  !       call declare_chemloss
  !       call chemloss_apply
  !       call free_chemloss
  !
  !       method
  !       ------
  !       subroutine declare_chemloss is called from trace0
  !       subroutine chemloss_apply   is called from source1
  !       subroutine free_chemloss    is called from trace_end
  !-----------------------------------------------------------------------

  ! Table of Contents:
  !
  !  module chemloss
  !    subroutine Chemloss_Init(status)
  !    end subroutine Chemloss_Init
  !    subroutine chemloss_apply(region)
  !    end subroutine chemloss_apply
  !    subroutine Chemloss_Done(budget_file,status)
  !    end subroutine Chemloss_Done
  !  end module chemloss
  !EOP
  use GO,                 only: gol, goErr, goPr, goLabel
!  use emission_data,      only: budrr_dat, plandr, emis2D
  use emission_data,      only: plandr, emis2D
  use emission_data,      only: sum_chemloss
  use chemloss_ch4_all,   only: declare_chemloss_ch4_all, free_chemloss_ch4_all, calc_chemloss_ch4_all, chemloss_apply_ch4_all
  use chemloss_ch4_ohcl,  only: declare_chemloss_ch4_ohcl, free_chemloss_ch4_ohcl, calc_chemloss_ch4_ohcl
  use emission_common,    only: declare_emission_averagers, free_emission_averagers, calc_emission_averagers, save_emission_averagers
  use emission_common,    only: EmisInputDir

  use enkf_common,        only: savestate_hdf

  private

  ! public routines:
  public   :: Chemloss_Init, chemloss_apply, Chemloss_Done

  ! --- const --------------------------------------

  character(len=*), parameter  ::  mname = 'chemloss'



contains

  subroutine Chemloss_Init(status)
    !BOP
    ! !ROUTINE: Chemloss_Init
    ! !DESCRIPTION:
    !
    !       purpose
    !       -------
    !       allocate space needed to handle the photochemical loss
    !
    !       interface
    !       ---------
    !       call declare_chemloss is called from trace0 (start and newmonth)
    !
    !       method
    !       ------
    !       dynamic allocation, depending on dimensions problem
    !       opening of daily input files
    !       reading monthly/yearly emissions
    ! 
    !---------------------------------------------------------------------   
    !EOP
    use dims,               only: im,jm,lm,newsrun,nregions
    use chem_param,         only: ntracet
    use budget_global,      only: nbud_vg,budrrg

    use ParTools,           only: nmembers, nmembers_per_pe, nmembersloc, maxtrac_per_pe
    use ParTools,           only: ntracetloc
    use GO,                 only: TrcFile, Init, Done, ReadRc
    use global_data,        only: rcfile
    implicit none
    ! --- in/out --------------------------------

    integer, intent(out)           ::  status

    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/Chemloss_Init'

    ! local
    integer                   :: region, imr, jmr, lmr

    type(TrcFile)        :: rcF

    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'emis.input.dir', EmisInputDir ,status ,default='./')
    IF_NOTOK_RETURN(status=1)
    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)

#ifdef with_budget
    ! on all PEs to be made available:
    do region=1,nregions
      imr = im(region) ;jmr = jm(region) ; lmr = lm(region)
!      allocate(budrr_dat(region)%emi(im(region),jm(region),nbud_vg,ntracet))
!      budrr_dat(region)%emi     = 0.0
      sum_chemloss(region)       = 0.0
    enddo
#endif


    call declare_chemloss_ch4_ohcl
    call declare_chemloss_ch4_all


    status=0

  end subroutine Chemloss_Init

  ! ---------------------------------

  subroutine chemloss_apply(region)
    use Dims,          only: okdebug
    use chem_param,    only : ntracet
    use ParTools,      only: ntracetloc, tracer_active, myid, root_t

    implicit none
    integer, intent(in)        :: region

    if(okdebug) write(*,*) 'start of chemloss_apply'
    if(ntracetloc == 0) return      !if no tracer on this processor do nothing

!    if(region == 1) then  !WP! only needed once, on 1x1 grid, coarsened array used after that
      call calc_chemloss_ch4_ohcl
      call calc_chemloss_ch4_all

!    endif

    call chemloss_apply_ch4_all(region)

    if(okdebug) write(*,*) 'End of adding chemloss ' 

  end subroutine chemloss_apply

  subroutine Chemloss_Done(budget_file,status)
    !
    !       purpose
    !       -------
    !       deallocate space needed to handle the chemical loss
    !
    !       interface
    !       ---------
    !       call free_chemloss is called from trace_end
    !
    !       method
    !       ------
    !       collecting budgets on coarser budget regions
    !       writing budget file
    !       deallocation of memory
    !       closing of input files
    !---------------------------------------------------------------------   
    use dims,          only : im,jm, nregions
    use chem_param,    only : ntracet

    use budget_global, only : nbudg, nbud_vg, budg_dat, apply_budget_global, budrrg
    use io_hdf,        only : DFACC_WRITE, io_write
#ifdef MPI
    use mpi_const, only: lmloc, tracer_active, ntracetloc, myid, root, my_real, mpi_sum, com_trac, ierr, root_t
#else
    use ParTools, only: lmloc, tracer_active, myid, root, ntracetloc, root_t
#endif    

    implicit none
    ! --- in/out --------------------------------

    integer, intent(out)           ::  status
    character(len=*)               :: budget_file

    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/Chemloss_Done'

    ! local
    integer                                  :: region, imr, jmr
    integer                                  :: i,j,n, nzone
!    real, dimension(nbudg, nbud_vg, ntracet) :: budrrg
    real, dimension(:,:,:,:),allocatable     :: budget4d

    integer                                  :: ifail,sfend, nsend, io, sfstart

#ifdef with_budget
    if(apply_budget_global) then

       !handle budget emission:

       if(myid == root) then
          io = sfstart(budget_file, DFACC_WRITE)
       endif

!       budrrg(:,:,:) = 0.0
!       do region = 1, nregions
!          allocate(budget4d(im(region),jm(region),nbud_vg,ntracet))
!#ifdef MPI
!          nsend = im(region)*jm(region)*nbud_vg*ntracet
!          call mpi_allreduce(budrr_dat(region)%emi,budget4d, &
!               nsend,my_real,mpi_sum,com_trac,ierr)
!#else
!          budget4d = budrr_dat(region)%emi
!#endif
!          call io_write(io,im(region),'im',jm(region),'jm', nbud_vg, 'nbud_vg', ntracet,'ntracet', &
!               budget4d ,'budemi_dat')
!          do n=1,ntracet 
!             do j=1,jm(region)
!                do i=1,im(region)
!                   nzone = budg_dat(region)%nzong(i,j)
!                   budrrg(nzone,:,n) = budrrg(nzone,:,n) + &
!                        budget4d(i,j,:,n) 
!                end do
!             end do!j
!          end do !
!          deallocate(budget4d)
!       enddo  ! regions

       if(myid == root) then
          call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
               ntracet,'ntracet',budrrg,'budrr')
          print *, 'Close budget file:', sfend(io)
       endif

    endif ! apply_budget_global

!    do region = 1, nregions
!       deallocate(budrr_dat(region)%emi)
!    enddo
#endif

   call free_chemloss_ch4_ohcl

    status=0

  end subroutine Chemloss_Done

end module chemloss
