module emission_ch4_all
  !-----------------------------------------------------------------------
  !       purpose
  !       -------
  !       perform ch4 emissions all
  !
  !         The convention for naming source input files
  !         obtained from Bergamaschi for 2002
  ! s0001=coal
  ! s0002=oil and gas
  ! s0003=enteric fermentation
  ! s0004=rice
  ! s0005=biomass burning
  ! s0006=waste
  ! s0007=wet lands
  ! s0008= wild animals?
  ! s0009= termites
  ! s0010=soils
  ! s0011=oceans
  !
  !       interface_
  !       ---------
  !       call emission_apply_all
  !
  !       method
  !       ------
  !       subroutine is called from its general parent routine apply_emission
  !-----------------------------------------------------------------------
  use chem_param ,         only: xmch4, xmc,ra
  use global_types,        only: emis_data
  use Dims,                only: nregions, nlon360,nlat180
  use emission_data,       only: do_add_2d, flux_to_gridbox_per_month
  use ParTools,            only: maxtrac_per_pe
  use emission_common,     only: ch4_ocean, ch4_soil, ch4_entferm, ch4_bburn, ch4_term, ch4_rice, ch4_waste, &
                                 ch4_wildan, ch4_wetl, ch4_coal, ch4_oilgas, ch4_all
  use emission_common,     only: ch4_all_m, ch4_all_n, flux_means_are_priors
  use enkf_common,         only: scalefac
  use enkf_common,         only: state_X     ! for future direct use of parameter values
  use ParTools,            only: nmembersloc ! to interpret state_X, scalefac, nmembersloc
  use enkf_constants, only: nparams_in_state,npar_ocn,npar_land,nreg_ocn,nreg_land

  implicit none

  private

  ! public routines:
  public   :: emission_apply_ch4_all, declare_emission_ch4_all, free_emission_ch4_all, calc_emission_ch4_all
  public   :: ch4_flux, emis_data, pseudo

  type(emis_data),dimension(nregions,maxtrac_per_pe),target       :: ch4_flux

  logical                                                         :: flux_unallocated=.true.
  logical                                                         :: pseudo=.false.

contains

  subroutine declare_emission_ch4_all
   use dims,       only: im,jm,nregions
   use ParTools,   only: ntracetloc
   implicit none
   integer                                          :: region, imr, jmr, n

   do region=1,nregions
     imr=im(region)
     jmr=jm(region)
     do n=1,ntracetloc
       allocate(ch4_flux(region,n)%surf(imr,jmr))
     end do
   enddo

  end subroutine declare_emission_ch4_all

  subroutine calc_emission_ch4_all
   use dims,       only: nlon360, nlat180, im,jm, sec_month, newsrun, idate, newday,nregions, itau, staggered, nread
   use toolbox,    only: coarsen_emission_2d
   use ParTools,   only: tracer_active, ntracetloc
   use chem_param, only: xmc
   use partools,   only : myid,root_t,nmembersloc
   implicit none
   integer                                          :: region, imr, jmr, n, mem, nreg
   integer, parameter                               :: add_field=0
   real, parameter                                  :: sf2=1.5

    if(mod(itau,nread) /= 0) return  ! only every nread hours

    !WP! put full flux into emission array for each member

!    if(myid==root_t) print*,"DEBUG calc emis",itau,sum(ch4_bio)*xmc/1.e3,sum(ch4_ocean)*xmc/1.e3,sum(ch4_ff),mod(itau+staggered,nread/2),nread 
!    scalefac=1.
    do mem=1,nmembersloc
     ch4_all=0.0
     do nreg=1,nreg_land

! create full emissions in mol/m2/s
       ch4_all=ch4_all+ch4_coal*scalefac(1+(nreg-1)*10,:,:,mem)+&
               ch4_oilgas*scalefac(2+(nreg-1)*10,:,:,mem)+&
               ch4_entferm*scalefac(3+(nreg-1)*10,:,:,mem)+&
               ch4_rice*scalefac(4+(nreg-1)*10,:,:,mem)+&
               ch4_bburn*scalefac(5+(nreg-1)*10,:,:,mem)+&
               ch4_waste*scalefac(6+(nreg-1)*10,:,:,mem)+&
               ch4_wetl*scalefac(7+(nreg-1)*10,:,:,mem)+&
               ch4_wildan*scalefac(8+(nreg-1)*10,:,:,mem)+&
               ch4_term*scalefac(9+(nreg-1)*10,:,:,mem)+&
               ch4_soil*scalefac(10+(nreg-1)*10,:,:,mem)
       print *,1+(nreg-1)*10,10+(nreg-1)*10
       print *,'total coal',sum(ch4_coal),sum(scalefac(1+(nreg-1)*10,:,:,mem))
       print *,'total oilgas',sum(ch4_oilgas),sum(scalefac(2+(nreg-1)*10,:,:,mem))
        print *,'total entferm',sum(ch4_entferm),sum(scalefac(3+(nreg-1)*10,:,:,mem))
        print *,'total rice',sum(ch4_rice),sum(scalefac(4+(nreg-1)*10,:,:,mem))
        print *,'total bburn',sum(ch4_bburn),sum(scalefac(5+(nreg-1)*10,:,:,mem))
        print *,'total waste',sum(ch4_waste),sum(scalefac(6+(nreg-1)*10,:,:,mem))
        print *,'total wetl',sum(ch4_wetl),sum(scalefac(7+(nreg-1)*10,:,:,mem))
        print *,'total wildan',sum(ch4_wildan),sum(scalefac(8+(nreg-1)*10,:,:,mem))
        print *,'total term',sum(ch4_term),sum(scalefac(9+(nreg-1)*10,:,:,mem))
     enddo
!     print *,npar_land,nreg_ocn,npar_ocn
     do nreg=npar_land*nreg_land+1,npar_land*nreg_land+nreg_ocn
        ch4_all=ch4_all+ch4_ocean*scalefac(nreg,:,:,mem) 
     enddo
        print *,'total all',sum(ch4_all)
        print *,'total ocean',sum(ch4_ocean),sum(scalefac(121,:,:,mem))
        stop

       ! accumulate weekly average
       if(flux_means_are_priors .eq. .FALSE.) then
         if(allocated(ch4_all_m)) then
          ch4_all_m = ch4_all_m+ch4_all
          ch4_all_n=ch4_all_n+1
         end if
       end if

       ch4_all=ch4_all*xmch4/1.e3 ! full emissions in kgC/m2/s
!       ch4_all=ch4_all*sf2        ! make priors 50% larger to test bias

 !      if(myid==root_t) print*,"DEBUG calc emis mem",mem,nmembersloc,sum(scalefac(:,:,mem)),sum(ch4_all)

       call flux_to_gridbox_per_month(ch4_all) ! to kgC/area/month
!       print *,'LB:total in emission_all',sum(ch4_all)*1000./1.e12*12.
!       stop
       call coarsen_emission_2d('ch4',360,180,ch4_all,ch4_flux(:,mem),add_field)

    end do

    ! accumulate weekly average
    if(flux_means_are_priors .eq. .TRUE.) then
       if(allocated(ch4_all_m)) then
          ch4_all_m=ch4_all_m+(ch4_bburn+ch4_ocean+ch4_soil+ch4_coal+ch4_oilgas+ch4_term+ &
                               ch4_waste+ch4_wetl+ch4_wildan+ch4_rice+ch4_entferm)
          ch4_all_n=ch4_all_n+1
       end if
    end if


  end subroutine calc_emission_ch4_all

  subroutine emission_apply_ch4_all(region)
  !
    use dims,          only: okdebug
    use ParTools,      only: tracer_active, ntracetloc, myid, ntracet_ar

    implicit none
    ! io        
    integer, intent(in)        :: region
    ! local
    integer                    :: n, offsetn=0
    ! start
    if(myid > 0) offsetn=sum(ntracet_ar(0:myid-1))
    do n=1,ntracetloc
!      print *,'total flux',sum(ch4_flux(region,n)%surf),ntracetloc,n
      call do_add_2d(region,offsetn+n,1,ch4_flux(region,n)%surf,ra(n),ra(n))
    enddo

    if(okdebug) write(*,*) 'end of emission_apply_ch4_all'

   end subroutine emission_apply_ch4_all

!==================================================================

   subroutine free_emission_ch4_all
   use dims,       only: nregions_max
   use ParTools,   only: ntracetloc
   implicit none
   integer                                          :: region, n

   do region=1,nregions
     do n=1,ntracetloc
       deallocate(ch4_flux(region,n)%surf)
     end do
   enddo


   end subroutine free_emission_ch4_all

end module emission_ch4_all
