
!### macros #####################################################
!
#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!#################################################################

module emission_ch4_bburn
  !-----------------------------------------------------------------------
  !       purpose
  !       -------
  !       perform ch4 biomass burning emissions 
  !
  !       EMISSIONS
  !          interface:     call emission_apply_bburn
  !          method:        subroutine is called from its general parent routine apply_emission
  !
  !       COVARIANCE
  !          interface:     call covariance_co2_compute_bburn
  !          method:        subroutine is called from MakePriorUncertainty in enkf_tools.F90
  !
  ! Table of contents:
  !   module emission_ch4_bburn
  !     subroutine declare_emission_ch4_bburn
  !     end subroutine declare_emission_ch4_bburn
  !     subroutine calc_emission_ch4_bburn
  !     end subroutine calc_emission_ch4_bburn
  !     subroutine emission_apply_ch4_bburn(region)
  !     end subroutine emission_apply_ch4_bburn
  !     subroutine free_emission_ch4_bburn
  !     end subroutine free_emission_ch4_bburn
  !   end module emission_ch4_bburn
  !
  !-----------------------------------------------------------------------

  use GO,                  only: gol, goErr, goPr, goLabel
  use Dims,                only: nregions, nlon360,nlat180
  use emission_data,       only: gridbox_per_sec_to_flux
  use global_types,        only: emis_data
  use partools,            only: nmembersloc
  use enkf_common,         only: scalefac
  use emission_common,     only: enkf_is_forward, enkf_is_inverse, EmisInputDir
  use emission_common,     only: ch4_bburn, ch4_bburn_m, ch4_bburn_n,flux_means_are_priors
  use emission_data,       only: do_add_2d, flux_to_gridbox_per_month
  use enkf_constants,      only: nparams_in_state,npar_ocn,npar_land,nreg_ocn,nreg_land

  implicit none

  ! --- const -------------------------
  character(len=*), parameter  ::  mname = 'emission_ch4_bburn'

  private

  ! public routines:
  public   :: declare_emission_ch4_bburn, free_emission_ch4_bburn, calc_emission_ch4_bburn, emission_apply_ch4_bburn

  ! ch4_bburn_regxmem is the result of the calculate routine.
  ! It holds the coarsened flux per ensemble member and region.
  type(emis_data),allocatable,dimension(:,:),target       :: ch4_bburn_regxmem

contains

  subroutine declare_emission_ch4_bburn

    use dims,       only: im,jm,nregions

    implicit none
    integer                            :: status=0
    integer                            :: region, imr, jmr, n

    character(len=*), parameter        :: rname = mname//'/declare_emission_ch4_bburn'

    if(enkf_is_forward()) then
!       print *, "DEBUG: declare_emission_ch4_bburn (forward)",nregions,nmembersloc
       allocate(ch4_bburn_regxmem(nregions, nmembersloc))

       do region=1,nregions
          imr=im(region)
          jmr=jm(region)
!          print *, "DEBUG: ",region, imr,jmr
          do n=1,nmembersloc
             allocate(ch4_bburn_regxmem(region,n)%surf(imr,jmr))
          end do
       enddo
    end if
  end subroutine declare_emission_ch4_bburn

  subroutine calc_emission_ch4_bburn

    use dims,          only: nlon360, nlat180, im,jm, sec_month, newsrun, idate, newmonth, sec_year,newday
    use io_save,       only: ReadFromFile,readtm3hdf
    use emission_data, only: chardate
    use chem_param,    only: xmch4,ra
    use global_data, only : inputdir_ch4
    use toolbox,       only: coarsen_emission_2d, escape_tm
    use go_string,     only: goNum2str

    implicit none
    integer                            :: status=0
    integer                            :: region, imr, jmr,lmr,irec,i,j
    integer, parameter                 :: annual_field = 0, rank2 = 2, level1 = 1
    character(len=10)                  :: cdate
    real                               :: year2month
    character (len=75)                           :: filename
    character (len=5)                            :: varname
    character (len=8)                            :: recname
    character(len=2),dimension(12),parameter     :: mlab=(/'01','02','03','04','05','06','07','08','09','10','11','12'/)
    character(len=5),parameter                   :: source_lab='S0005'

    real,dimension(nlon360,nlat180)    :: ch4_bburn_mem ! flux for a given ensemble member
    integer, parameter                 :: add_field=0
    integer                            :: mem,nreg

    character(len=*), parameter        :: rname = mname//'/calc_emission_ch4_bburn'

  if(newday) then
      filename=trim(inputdir_ch4)//'day_CH4_'//source_lab//'_'//mlab(idate(2))//'.hdf'
      varname=source_lab
      irec=idate(3)-1
      if (idate(2).eq.2.AND.idate(3).gt.28) irec=28-1
      print *,filename,'  ',varname,'  ',irec
      call readtm3hdf(filename,rank2,nlon360,nlat180,level1,irec,ch4_bburn,varname)  ! incoming units are kgCH4/gridbox/sec

       if (status /= 0) call escape_tm('stop in '//rname)
       call gridbox_per_sec_to_flux(ch4_bburn)   ! to kgC/m2/s
       ch4_bburn=ch4_bburn*1.e3/xmch4  ! to mol/m2/s
       
       ! accumulate weekly average
       !
       if(flux_means_are_priors .eq. .TRUE.) then

       if(allocated(ch4_bburn_m)) then
          ch4_bburn_m=ch4_bburn_m+ch4_bburn ! mol/m2/s
          ch4_bburn_n=ch4_bburn_n+1
       endif

       ! The rest only useful to the forward code
       if(enkf_is_inverse()) return

       ! Apply net flux scaling by ensemble member, change units, and
       ! coarsen to the regions.

       do mem=1,nmembersloc
          ch4_bburn_mem=ch4_bburn ! mol/m2/s (this is where parameters would be applied for optimized fluxes)
        do nreg=1,nreg_land
         do i=1,nlon360
          do j=1,nlat180
           if (scalefac(5+(nreg-1)*10,i,j,mem).ne.0.0) then
            ch4_bburn_mem(i,j)=ch4_bburn_mem(i,j)*scalefac(5+(nreg-1)*10,i,j,mem)
           endif
          enddo
         enddo
        enddo

          ! accumulate weekly average
          if(flux_means_are_priors .eq. .FALSE.) then
             ch4_bburn_m=ch4_bburn_m+ch4_bburn_mem ! mol/m2/s
             ch4_bburn_n=ch4_bburn_n+1
          endif
          ch4_bburn_mem=ch4_bburn_mem*xmch4/1.e3 ! from mol/m2/s to kgC/m2/s
          call flux_to_gridbox_per_month(ch4_bburn_mem) ! to kgC/area/month
          call coarsen_emission_2d('ch4_bburn',360,180,ch4_bburn_mem,ch4_bburn_regxmem(:,mem),add_field)
       enddo

    endif
  endif

  end subroutine calc_emission_ch4_bburn

  subroutine emission_apply_ch4_bburn(region)

    ! This routine should only be called by the forward enkf code.  It
    ! applies the bburn ch4 flux by member and region to the ibb
    ! tracer.  In the inverse code, the calculated flux from each
    ! module is applied to the (single) ch4 tracer.

    use dims,          only: okdebug
    use ParTools,      only: tracer_active
    use chem_param, only         : ibb, xmc, xmch4, ra


    implicit none

    integer, intent(in)        :: region
    integer                    :: mem

    character(len=*), parameter        :: rname = mname//'/emission_apply_ch4_bburn'

    if(enkf_is_forward()) then
#ifdef MPI
#else
       if(ibb .gt. 0) then
          do mem=1,nmembersloc
             call do_add_2d(region,ibb,1,ch4_bburn_regxmem(region,mem)%surf,ra(ibb),ra(ibb))
          enddo
       endif
#endif
    endif

    if(okdebug) write(*,*) 'end of emission_apply_ch4_bburn'

  end subroutine emission_apply_ch4_bburn

  subroutine free_emission_ch4_bburn
    use dims,        only: nregions

    implicit none
    integer               :: region,n

    character(len=*), parameter        :: rname = mname//'/free_emission_ch4_bburn'

    if(enkf_is_forward()) then
       do region=1,nregions
          do n=1,nmembersloc
             deallocate(ch4_bburn_regxmem(region,n)%surf)
          end do
       enddo

       deallocate(ch4_bburn_regxmem)
    end if

  end subroutine free_emission_ch4_bburn

end module emission_ch4_bburn
