!### macros #####################################################
!
#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!#################################################################

module emission
  !BOP
  ! !DESCRIPTION:
  !-----------------------------------------------------------------------
  !       purpose
  !       -------
  !       perform emissions needed for TM5 CBM4 version
  !
  !       interface
  !       ---------
  !       call declare_emission
  !       call emission_apply
  !       call free_emission
  !
  !       method
  !       ------
  !       subroutine declare_emission is called from trace0
  !       subroutine emission_apply   is called from source1
  !       subroutine free_emission    is called from trace_end
  !-----------------------------------------------------------------------

  ! Table of Contents:
  !
  !  module emission
  !    subroutine Emis_Init(status)
  !    end subroutine Emis_Init
  !    subroutine emission_apply(region)
  !    end subroutine emission_apply
  !    subroutine Emis_Done(budget_file,status)
  !    end subroutine Emis_Done
  !  end module emission
  !EOP
  use GO,                 only: gol, goErr, goPr, goLabel
 ! use emission_data,      only: budemi_dat, plandr, emis2D
  use emission_data,      only: plandr, emis2D
  use emission_data,      only: sum_emission
  use emission_common,    only: declare_emission_averagers, free_emission_averagers, calc_emission_averagers, save_emission_averagers
  use emission_common,    only: EmisInputDir
  use emission_ch4_coal  , only: declare_emission_ch4_coal, free_emission_ch4_coal, calc_emission_ch4_coal, emission_apply_ch4_coal
  use emission_ch4_oilgas, only: declare_emission_ch4_oilgas, free_emission_ch4_oilgas, calc_emission_ch4_oilgas, emission_apply_ch4_oilgas
  use emission_ch4_entferm, only: declare_emission_ch4_entferm, free_emission_ch4_entferm, calc_emission_ch4_entferm, emission_apply_ch4_entferm
  use emission_ch4_rice , only: declare_emission_ch4_rice, free_emission_ch4_rice, calc_emission_ch4_rice, emission_apply_ch4_rice
  use emission_ch4_bburn , only: declare_emission_ch4_bburn, free_emission_ch4_bburn, calc_emission_ch4_bburn, emission_apply_ch4_bburn
  use emission_ch4_waste , only: declare_emission_ch4_waste, free_emission_ch4_waste, calc_emission_ch4_waste, emission_apply_ch4_waste
  use emission_ch4_wetl , only: declare_emission_ch4_wetl, free_emission_ch4_wetl, calc_emission_ch4_wetl, emission_apply_ch4_wetl
  use emission_ch4_wildan , only: declare_emission_ch4_wildan, free_emission_ch4_wildan, calc_emission_ch4_wildan, emission_apply_ch4_wildan
  use emission_ch4_term , only: declare_emission_ch4_term, free_emission_ch4_term, calc_emission_ch4_term, emission_apply_ch4_term
  use emission_ch4_soil , only: declare_emission_ch4_soil, free_emission_ch4_soil, calc_emission_ch4_soil, emission_apply_ch4_soil
  use emission_ch4_ocean , only: declare_emission_ch4_ocean, free_emission_ch4_ocean, calc_emission_ch4_ocean, emission_apply_ch4_ocean

  use enkf_common,        only: savestate_hdf

  private

  ! public routines:
  public   :: Emis_Init, emission_apply, Emis_Done

  ! --- const --------------------------------------

  character(len=*), parameter  ::  mname = 'emission'



contains

  subroutine Emis_Init(status)
    !BOP
    ! !ROUTINE: Emis_Init
    ! !DESCRIPTION:
    !
    !       purpose
    !       -------
    !       allocate space needed to handle the emissions
    !
    !       interface
    !       ---------
    !       call declare_emission is called from trace0 (start and newmonth)
    !
    !       method
    !       ------
    !       dynamic allocation, depending on dimensions problem
    !       opening of daily input files
    !       reading monthly/yearly emissions
    ! 
    !---------------------------------------------------------------------   
    !EOP
    use dims,               only: im,jm,lm,newsrun,nregions
    use chem_param,         only: ntracet
    use budget_global,      only: nbud_vg
    use emission_common,    only: flux_means_are_priors

    use ParTools,           only: nmembers, nmembers_per_pe, nmembersloc, maxtrac_per_pe
    use ParTools,           only: ntracetloc
    use GO,                 only: TrcFile, Init, Done, ReadRc
    use global_data,        only: rcfile
    implicit none
    ! --- in/out --------------------------------

    integer, intent(out)           ::  status

    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/Emis_Init'

    ! local
    integer                   :: region, imr, jmr, lmr

    type(TrcFile)        :: rcF

    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'flux_means_are_priors', flux_means_are_priors ,status ,default=.false.)
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'emis.input.dir', EmisInputDir ,status ,default='./')
    IF_NOTOK_RETURN(status=1)
    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)

    call declare_emission_ch4_coal
    call declare_emission_ch4_oilgas
    call declare_emission_ch4_entferm
    call declare_emission_ch4_rice
    call declare_emission_ch4_bburn
    call declare_emission_ch4_waste
    call declare_emission_ch4_wetl
    call declare_emission_ch4_wildan
    call declare_emission_ch4_term
    call declare_emission_ch4_soil
    call declare_emission_ch4_ocean


    ! initialize the average flux accumulators
    call declare_emission_averagers

#ifdef with_budget
    ! on all PEs to be made available:
    do region=1,nregions
       imr = im(region) ;jmr = jm(region) ; lmr = lm(region)
!       allocate(budemi_dat(region)%emi(im(region),jm(region),nbud_vg,ntracet))
!       budemi_dat(region)%emi     = 0.0
       sum_emission(region)       = 0.0
    enddo
#endif

    status=0

  end subroutine Emis_Init

  ! ---------------------------------

  subroutine emission_apply(region)
    use Dims,          only: okdebug
    use chem_param,    only : ntracet
    use ParTools,      only: ntracetloc, tracer_active, myid, root_t

    implicit none
    integer, intent(in)        :: region

    if(okdebug) write(*,*) 'start of emission_apply'
    if(ntracetloc == 0) return      !if no tracer on this processor do nothing

!<<<<<<< emission__fwd.F90
!    call calc_emission_ch4_coal
!    call calc_emission_ch4_oilgas
!    call calc_emission_ch4_entferm
!    call calc_emission_ch4_rice
!    call calc_emission_ch4_bburn
!    call calc_emission_ch4_waste
!    call calc_emission_ch4_wetl
!    call calc_emission_ch4_wildan
!    call calc_emission_ch4_term
!    call calc_emission_ch4_soil
!    call calc_emission_ch4_ocean
!=======
    if(region == 1) then  !WP! only needed once, on 1x1 grid, coarsened array used after that
    call calc_emission_ch4_coal
    call calc_emission_ch4_oilgas
    call calc_emission_ch4_entferm
    call calc_emission_ch4_rice
    call calc_emission_ch4_bburn
    call calc_emission_ch4_waste
    call calc_emission_ch4_wetl
    call calc_emission_ch4_wildan
    call calc_emission_ch4_term
    call calc_emission_ch4_soil
    call calc_emission_ch4_ocean
    endif

    call emission_apply_ch4_coal(region)
    call emission_apply_ch4_oilgas(region)
    call emission_apply_ch4_entferm(region)
    call emission_apply_ch4_rice(region)
    call emission_apply_ch4_bburn(region)
    call emission_apply_ch4_waste(region)
    call emission_apply_ch4_wetl(region)
    call emission_apply_ch4_wildan(region)
    call emission_apply_ch4_term(region)
    call emission_apply_ch4_soil(region)
    call emission_apply_ch4_ocean(region)

    if(okdebug) write(*,*) 'End of adding emission ' 

  end subroutine emission_apply

  subroutine Emis_Done(budget_file,status)
    !
    !       purpose
    !       -------
    !       deallocate space needed to handle the emissions
    !
    !       interface
    !       ---------
    !       call free_emission is called from trace_end
    !
    !       method
    !       ------
    !       collecting budgets on coarser budget regions
    !       writing budget file
    !       deallocation of memory
    !       closing of input files
    !---------------------------------------------------------------------   
    use dims,          only : im,jm, nregions
    use chem_param,    only : ntracet

    use budget_global, only : nbudg, nbud_vg, budg_dat, apply_budget_global,budemig
    use io_hdf,        only : DFACC_WRITE, io_write
#ifdef MPI
    use mpi_const, only: lmloc, tracer_active, ntracetloc, myid, root, my_real, mpi_sum, com_trac, ierr, root_t
#else
    use ParTools, only: lmloc, tracer_active, myid, root, ntracetloc, root_t
#endif    

    implicit none
    ! --- in/out --------------------------------

    integer, intent(out)           ::  status
    character(len=*)               :: budget_file

    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/Emis_Done'

    ! local
    integer                                  :: region, imr, jmr
    integer                                  :: i,j,n, nzone
!    real, dimension(nbudg, nbud_vg, ntracet) :: budemig
!    real, dimension(:,:,:,:),allocatable     :: budget4d

    integer                                  :: ifail,sfend, nsend, io, sfstart

#ifdef with_budget
    if(apply_budget_global) then

       !handle budget emission:

       if(myid == root) then
          io = sfstart(budget_file, DFACC_WRITE)
       endif

!       budemig(:,:,:) = 0.0
!       do region = 1, nregions
!          allocate(budget4d(im(region),jm(region),nbud_vg,ntracet))
!#ifdef MPI
!          nsend = im(region)*jm(region)*nbud_vg*ntracet
!          call mpi_allreduce(budemi_dat(region)%emi,budget4d, &
!               nsend,my_real,mpi_sum,com_trac,ierr)
!#else
!          budget4d = budemi_dat(region)%emi
!#endif
!          call io_write(io,im(region),'im',jm(region),'jm', nbud_vg, 'nbud_vg', ntracet,'ntracet', &
!               budget4d ,'budemi_dat')
!          do n=1,ntracet 
!             do j=1,jm(region)
!                do i=1,im(region)
!                   nzone = budg_dat(region)%nzong(i,j)
!                   budemig(nzone,:,n) = budemig(nzone,:,n) + &
!                        budget4d(i,j,:,n) 
!                end do
!             end do!j
!          end do !
!          deallocate(budget4d)
!       enddo  ! regions

       if(myid == root) then
          call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
               ntracet,'ntracet',budemig,'budemi')
          print *, 'Close budget file:', sfend(io)
       endif

    endif ! apply_budget_global

!    do region = 1, nregions
!       deallocate(budemi_dat(region)%emi)
!    enddo
#endif
    call calc_emission_averagers
    call save_emission_averagers(savestate_hdf)

   call free_emission_ch4_coal
   call free_emission_ch4_oilgas
   call free_emission_ch4_entferm
   call free_emission_ch4_rice
   call free_emission_ch4_bburn
   call free_emission_ch4_waste
   call free_emission_ch4_wetl
   call free_emission_ch4_wildan
   call free_emission_ch4_term
   call free_emission_ch4_soil
   call free_emission_ch4_ocean

    call free_emission_averagers

    status=0

  end subroutine Emis_Done

end module emission
