module tmm_mf

  use GO, only : TDate
  use Grid, only : TllGridInfo, TggGridInfo, TshGridInfo, TshGrid, TLevelInfo

#include "tmm.h"

#ifdef with_hdf  
  use tmm_mf_hdf  , only : TMeteoFile_hdf
#endif
#ifdef with_grib
  use tmm_mf_grib , only : TMeteoFile_grib
#endif
#ifdef with_prism  
  use tmm_mf_prism, only : TMeteoFile_prism
#endif
  
  implicit none
  
  ! --- in/out -------------------------------------
   
  private
 
  public   ::  TMeteoFile
  public   ::  Init, Done
  public   ::  Opened
  public   ::  Setup
  public   ::  CheckTime, CheckParam
  public   ::  ReadRecord
  public   ::  ReadEqvLatStuff
  
  
  ! --- const ---------------------------------------
  
  character(len=*), parameter  ::  mname = 'module tmm_mf'

  ! --- types --------------------------------------
  
  type TMeteoFile
    ! opened yet ?
    logical                    ::  opened
    ! meteo archive keys:
    character(len=256)         ::  dir_disk
    character(len=32)          ::  sourcekey
    ! parameter keys for fields in this file
    character(len=256)         ::  paramkeys
    ! time range for which file is valid
    type(TDate)                ::  t1, t2
    ! quick and dirty testing ?
    logical                    ::  qad
    !
    ! access to current meteo file
    !
    character(len=10)          ::  filetype
    character(len=256)         ::  filename
    character(len=256)         ::  spm_filename
#ifdef with_hdf
    type(TMeteoFile_hdf)       ::  mf_hdf         ! hdf file
#endif
#ifdef with_grib
    type(TMeteoFile_grib)      ::  mf_grib        ! grib file
#endif
#ifdef with_prism  
    type(TMeteoFile_prism)     ::  mf_prism       ! prism file
#endif
  end type TMeteoFile


  ! --- interfaces -------------------------------
  
  interface Init
    module procedure mf_Init
  end interface

  interface Done
    module procedure mf_Done
  end interface

  interface Opened
    module procedure mf_Opened
  end interface

  interface Setup
    module procedure mf_Setup
  end interface

  interface CheckTime
    module procedure mf_CheckTime
  end interface

  interface CheckParam
    module procedure mf_CheckParam
  end interface

  interface ReadRecord
    module procedure mf_ReadRecord
  end interface

  interface ReadEqvLatStuff
    module procedure mf_ReadEqvLatStuff
  end interface


contains


  ! ===========================================================
  !
  !   init/done
  !
  ! ===========================================================
  

  subroutine mf_Init( mf, status )
  
    use GO, only : NewDate
                       
    ! --- begin -------------------------------------------
    
    type(TMeteoFile), intent(out)           ::  mf
    integer, intent(out)                    ::  status
    
    ! --- const -------------------------------------------
    
    character(len=*), parameter ::  name = mname//', mf_Init'
    
    ! --- begin -------------------------------------------

    ! file not opened yet
    mf%opened = .false.
    
    ! ok
    status = 0
    
  end subroutine mf_Init
  
  
  ! ***
  
  
  subroutine mf_Done( mf, status )
  
    use GO, only : goSystem
    use Grid, only : Done

#ifdef with_hdf
    use tmm_mf_hdf  , only : Done
#endif
#ifdef with_grib
    use tmm_mf_grib , only : Done
#endif
#ifdef with_prism  
    use tmm_mf_prism, only : Done
#endif
  
    ! --- begin -------------------------------------------
    
    type(TMeteoFile), intent(inout)         ::  mf
    integer, intent(out)                    ::  status
    
    ! --- const -------------------------------------------
    
    character(len=*), parameter ::  name = mname//', mf_Done'
    
    ! --- begin -------------------------------------------

    ! close file if necessary
    if ( mf%opened ) then
      select case ( mf%filetype )
#ifdef with_hdf  
        case ( 'hdf' )
          call Done( mf%mf_hdf, status )
          if (status/=0) then; write (*,'("ERROR in ",a)') name; status=1; return; end if
#endif
#ifdef with_grib  
        case ( 'grib' )
          call Done( mf%mf_grib, status )
          if (status/=0) then; write (*,'("ERROR in ",a)') name; status=1; return; end if
#endif
#ifdef with_prism  
        case ( 'prism' )
          call Done( mf%mf_prism, status )
          if (status/=0) then; write (*,'("ERROR in ",a)') name; status=1; return; end if
#endif
        case default
          write (*,'("ERROR - unsupported filetype `",a,"`")') trim(mf%filetype)
          write (*,'("ERROR in ",a)') name; status=1; return
      end select
      mf%opened = .false.
    end if
    
    ! ok
    status = 0
    
  end subroutine mf_Done
  
  
  ! ***
  
  
  logical function mf_Opened( mf )

    ! --- begin -------------------------------------------
    
    type(TMeteoFile), intent(in)         ::  mf
    
    ! --- begin -------------------------------------------
    
    mf_Opened = mf%opened
    
  end function mf_Opened
    

  ! ===========================================================
  !
  !   open meteo file
  !
  ! ===========================================================
  

  !
  ! Open the meteo file that contains the field specified by
  ! sourcekey, parameter key, time,
  ! or do nothing if the requested file has been opened already.
  !
  ! <sourcekey> = <sourcetype>:<sourcename>
  !
  !    tmpp:od-fc-ml60-glb3x2
  !   tmppS:od-fc-ml60-glb3x2
  !    grib:od-fc-ml60-glb3x2
  !   prism:
  !
  
  subroutine mf_Setup( mf, sourcekey, paramkey, t1, t2, fcday, &
                       shellscript, rcfilename, status )

    use GO, only : goSplitLine, goReadFromLine
    use GO, only : goSystem
    use GO, only : TrcFile, Init, Done, ReadRc
    use GO, only : TDate, IncrDate, Get, NewDate, PrintDate, &
                   Operator(+), Operator(-), Operator(/)

#ifdef with_hdf  
    use tmm_mf_hdf  , only : Init
#endif
#ifdef with_grib  
    use tmm_mf_grib , only : Init
#endif
#ifdef with_prism  
    use tmm_mf_prism, only : Init
#endif
    
    ! --- in/out -------------------------------------

    type(TMeteoFile), intent(inout)        ::  mf
    character(len=*), intent(in)           ::  sourcekey
    character(len=*), intent(in)           ::  paramkey
    type(TDate), intent(in)                ::  t1, t2
    integer, intent(in)                    ::  fcday
    character(len=*), intent(in)           ::  shellscript
    character(len=*), intent(in)           ::  rcfilename
    integer, intent(inout)                 ::  status
    
    ! --- const -------------------------------------
    
    character(len=*), parameter ::  name = mname//', mf_Setup'
    
    ! name of info file:
    character(len=*), parameter  ::  infofilename = 'tmm_info.rc'
  
    ! --- local -------------------------------------
    
    character(len=10)        ::  sourcetype
    character(len=40)        ::  sourcename
    
    character(len=256)       ::  command
    integer                  ::  year1, month1, day1, hour1
    integer                  ::  year2, month2, day2, hour2
    type(TrcFile)            ::  infofile

    character(len=100)       ::  sourcename2
    character(len=10)        ::  mclass
    character(len=10)        ::  mtype
    character(len=10)        ::  mlevs
    character(len=20)        ::  mgrid
    character(len=10)        ::  filekey
    character(len=10)        ::  treskey
    logical                  ::  with_spm
    logical                  ::  constant
    type(TDate)              ::  tfc
    
    character(len=1)         ::  sep

    ! --- begin -------------------------------------
    
    ! store source key:
    mf%sourcekey = trim(sourcekey)
    
    ! split source key in type and name:
    call goSplitLine( sourcekey, sourcetype, ':', sourcename )
    
    ! usually, meteo is storred in file;
    ! for PRISM project, meteo is in the memory ...
    select case ( sourcetype )
    
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! hdf day files on file system
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case ( 'tmpp' )

        ! read some settings
        call Init( infofile, rcfilename )
        call ReadRc( infofile, 'tmm.qad', mf%qad      )
        call ReadRc( infofile, 'tmm.dir', mf%dir_disk )
        call ReadRc( infofile, 'tmm.sep', sep         )
        call Done( infofile )

        ! time range:
        call Get( t1, year=year1, month=month1, day=day1, hour=hour1 )
        if ( hour1 < 21 ) then
          mf%t1 = NewDate( year=year1, month=month1, day=day1, hour=0 )
          mf%t1 = mf%t1 - IncrDate(hour=3)
        else
          mf%t1 = NewDate( year=year1, month=month1, day=day1, hour=21 )
        end if
        mf%t2 = mf%t1 + IncrDate(hour=24)
        call Get( mf%t2, year=year2, month=month2, day=day2, hour=hour2 )

        !call PrintDate( '     fields valid from : ', mf%t1 )
        !call PrintDate( '                  to   : ', mf%t2 )

        ! file type and name:
        !   od-fc-2000-01-ml1-eur1x1-spm_20000111_00p06.hdf
        ! * set type:
        mf%filetype = 'hdf'
        ! * split sourcename
        sourcename2 = sourcename
        call goReadFromLine( sourcename2, mclass, sep='-' )
        call goReadFromLine( sourcename2, mtype , sep='-' )
        call goReadFromLine( sourcename2, mlevs , sep='-' )
        call goReadFromLine( sourcename2, mgrid , sep='-' )
        ! * other
        treskey = '21p06'
        with_spm = .true.
        ! * overwrite some fields:
        select case ( paramkey )
          case ( 'sp', 'pu', 'pv' )
            filekey = 'uvsp'
            mf%paramkeys = '-sp-pu-pv-'
          case ( 'T' )
            filekey = 't'
            mf%paramkeys = '-T-'
          case ( 'Q' )
            filekey = 'q'
            mf%paramkeys= '-Q-'
          case ( 'CLWC', 'CIWC', 'CC', 'CCO', 'CCU' )
            filekey = 'cld'
            mf%paramkeys= '-CLWC-CIWC-CC-CCO-CCU-'
          case ( 'eu', 'ed', 'du', 'dd', 'cloud_base', 'cloud_top', 'cloud_lfs' )
            filekey = 'sub'
            mf%paramkeys= '-eu-ed-du-dd-cloud_base-cloud_top-cloud_lfs-'
          case ( 'oro', 'lsm', 'sr_ecm', 'sr_ols', 'sr_mer', &
                 'cvl', 'cvh', &
                 'tv01', 'tv02', 'tv03', 'tv04', 'tv05', &
                 'tv06', 'tv07', 'tv08', 'tv09', 'tv10', &
                 'tv11', 'tv12', 'tv13', 'tv14', 'tv15', &
                 'tv16', 'tv17', 'tv18', 'tv19', 'tv20', &
                 'swvl1', &
                 'al', 'lsrh', 'ci', '10fg', 'u10m', 'v10m', 'sd', &
                 'lsp', 'cp', 'sf', 'sshf', 'slhf', 'blh', &
                 't2m', 'd2m', 'ssr', 'sstr', 'src', 'raero', 'ustar' )
            mlevs = 'sfc'
            mgrid = 'glb1x1'
            filekey = 'surf'
            treskey = '21p03'
            mf%paramkeys= '-'
            mf%paramkeys= trim(mf%paramkeys)//'oro-lsm-sr_ecm-sr_ols-sr_mer-'
            mf%paramkeys= trim(mf%paramkeys)//'cvl-cvh-'
            mf%paramkeys= trim(mf%paramkeys)//'tv01-tv02-tv03-tv04-tv05-'
            mf%paramkeys= trim(mf%paramkeys)//'tv06-tv07-tv08-tv09-tv10-'
            mf%paramkeys= trim(mf%paramkeys)//'tv11-tv12-tv13-tv14-tv15-'
            mf%paramkeys= trim(mf%paramkeys)//'tv16-tv17-tv18-tv19-tv20-'
            mf%paramkeys= trim(mf%paramkeys)//'swvl1-'
            mf%paramkeys= trim(mf%paramkeys)//'al-lsrh-ci-10fg-u10m-v10m-sd-'
            mf%paramkeys= trim(mf%paramkeys)//'lsp-cp-sf-sshf-slhf-blh-'
            mf%paramkeys= trim(mf%paramkeys)//'t2m-d2m-ssr-sstr-src-raero-ustar-'
            with_spm = .false.
          case ( 'spm' )
            mlevs = 'ml1'
            filekey = 'spm'
            treskey = '00p06'
            mf%paramkeys= '-spm-'
          case default
            write (*,'("ERROR - unsupported tmpp paramkey `",a,"`")') paramkey
            write (*,'("ERROR in ",a)') name; status=1; return
        end select
        ! * create filenames
        write (mf%filename,'(a,"/",a,a,a,a,i4.4,a,i2.2,a,a,a,a,a,a,"_",i4.4,i2.2,i2.2,"_",a,".hdf")') &
            trim(mf%dir_disk), &
            trim(mclass), sep, trim(mtype), sep, year2, sep, month2, sep, trim(mlevs), sep, trim(mgrid), sep, &
            trim(filekey), year2, month2, day2, trim(treskey)
        if ( with_spm ) then
          write (mf%spm_filename,'(a,"/",a,a,a,a,i4.4,a,i2.2,a,a,a,a,a,a,"_",i4.4,i2.2,i2.2,"_",a,".hdf")') &
              trim(mf%dir_disk), &
              trim(mclass), sep, trim(mtype), sep, year2, sep, month2, sep, 'ml1', sep, trim(mgrid), sep, &
              'spm', year2, month2, day2, '00p06'
        else
          mf%spm_filename = 'none'
        endif

        ! open file(s) :
        select case ( mf%filetype )
#ifdef with_hdf
          case ( 'hdf' )
            !WP!write (*,'("    open hdf file ...")')
            call Init( mf%mf_hdf, mf%filename, mf%spm_filename, status )
            if (status/=0) then; write (*,'("ERROR in ",a)') name; status=1; return; end if
#endif
          case default
            write (*,'("ERROR - unsupported filetype `",a,"`")') trim(mf%filetype)
            write (*,'("ERROR -   sourcetype : ",a)') trim(sourcetype)
            write (*,'("ERROR in ",a)') name; status=1; return
        end select


      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! meteo on file system: search online
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case ( 'tmppS' )

        ! Ask a shell script to return:
        !  o file name
        !  o file type
        !  o time range
        !  o ...
        ! This information is written to a text file, whos name is
        ! one of the arguments to the script.
        ! Sample command:
        !    tmm_getmeteo <infofile> <date>

        ! extract time parameters:
        call Get( t1, year=year1, month=month1, day=day1, hour=hour1 )
        call Get( t2, year=year2, month=month2, day=day2, hour=hour2 )

        ! write the command:
        !
        !  tmm_getmeteo  tmm_info_uvsp.rc  od-fc-ml60-glb3x2  pu   \
        !                2002123121  2003010103  1  \
        !                tm5pp.rc
        !
        write (command,'(a,3(" ",a),2(i5.4,3i2.2),i3," ",a)') &
                 trim(shellscript), &
                 trim(infofilename), trim(sourcename), trim(paramkey), &
                 year1, month1, day1, hour1, &
                 year2, month2, day2, hour2, &
                 fcday, &
                 trim(rcfilename)

        ! submit the command:
        !print *, trim(command)
        call goSystem( trim(command), status )
        if ( status /= 0 ) then
          write (*,'("ERROR - from system call :")')
          write (*,'("          ",a)') trim(command)
          write (*,'("ERROR -   status : ",i8)') status
          write (*,'("ERROR in ",a)') name; status = 1; return
        end if

        !
        ! Example of the info file:
        !
        !  ! Infofile for TM meteo file sequence.
        !  ! Comment lines start with '!'.
        !
        !  ! file key, for example 'uvsp' ; might be used for adhoc stuff:
        !  file.key : uvsp
        !
        !  ! parameter list, for example '-ps-pu-pv-'
        !  paramlist : -ps-pu-pv-
        !
        !  ! time range of fields in file:
        !  timerange.start  : 2002, 12, 31, 21
        !  timerange.finish : 2003, 01, 01, 21
        !
        !  ! target file type (hdf,gb)
        !  file.type : hdf
        !
        !  ! target file name:
        !  file.name  : /data/TM/meteo/od/fc/2003/01/ml60/gl3x2/uvsp_20030101_21p06.hdf
        !

        ! Open the info file as a rc file:
        write (*,'("  open info file ...")')
        call Init( infofile, infofilename )

        ! parameters
        call ReadRc( infofile, 'paramlist', mf%paramkeys )

        ! time range:
        call ReadRc( infofile, 'timerange.start' , year1, month1, day1, hour1 )
        call ReadRc( infofile, 'timerange.finish', year2, month2, day2, hour2 )
        mf%t1 = NewDate( year=year1, month=month1, day=day1, hour=hour1 )
        mf%t2 = NewDate( year=year2, month=month2, day=day2, hour=hour2 )
        !call PrintDate( '     fields valid from : ', mf%t1 )
        !call PrintDate( '                  to   : ', mf%t2 )

        ! open file:
        call ReadRc( infofile, 'file.type', mf%filetype )
        call ReadRc( infofile, 'file.name', mf%filename )
        call ReadRc( infofile, 'spm.file.name', mf%spm_filename )
        select case ( mf%filetype )
#ifdef with_hdf  
          case ( 'hdf' )
            !WP!write (*,'("    open hdf file ...")')
            call Init( mf%mf_hdf, mf%filename, mf%spm_filename, status )
            if (status/=0) then; write (*,'("ERROR in ",a)') name; status=1; return; end if
#endif
          case default
            write (*,'("ERROR - unsupported filetype `",a,"`")') trim(mf%filetype)
            write (*,'("ERROR in ",a)') name; status=1; return
        end select

        ! close info file:
        call Done( infofile)

        ! remove info file name if necessary:
        write (command,'("/bin/rm -f ",a)') trim(infofilename)
        call goSystem( command, status )
        if ( status /= 0 ) then
          write (*,'("ERROR - from system call :")')
          write (*,'("          ",a)') trim(command)
          write (*,'("ERROR -   status     : ",i4)') status
          write (*,'("ERROR -   sourcetype : ",a)') trim(sourcetype)
          write (*,'("ERROR in ",a)') name; status = 1; return
        end if

      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! grib files (tostie)
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      case ( 'grib' )

        write (*,'("             fcday : ",i2)') fcday

        ! read some settings
        call Init( infofile, rcfilename )
        call ReadRc( infofile, 'tmm.buf.dir', mf%dir_disk )
        call Done( infofile )
        
        ! date in filename is day that 12:00 fc started
        call Get( t1, year=year1, month=month1, day=day1 )
        tfc = NewDate( year=year1, month=month1, day=day1, hour=12 ) - IncrDate(day=fcday)

        call PrintDate( '             fcday : ', tfc )
        call Get( tfc, year=year1, month=month1, day=day1 )

        ! time range tfc+[0,240]
        mf%t1 = tfc
        mf%t2 = mf%t1 + IncrDate(hour=240)

        !call PrintDate( '     fields valid from : ', mf%t1 )
        !call PrintDate( '                  to   : ', mf%t2 )

        ! file type and name:
        !   od-fc-2000-01-ml60-glb1x1-T_20000111_00p03.hdf
        ! * set type:
        mf%filetype = 'grib'
        ! * split sourcename
        sourcename2 = sourcename
        call goReadFromLine( sourcename2, mclass, sep='-' )
        call goReadFromLine( sourcename2, mtype , sep='-' )
        call goReadFromLine( sourcename2, mlevs , sep='-' )
        call goReadFromLine( sourcename2, mgrid , sep='-' )
        ! * other
        treskey = '00p03'
        with_spm = .true.
        constant = .false.
        ! * overwrite some fields:
        select case ( paramkey )
          case ( 'T' )
            filekey = 'T'
            mf%paramkeys = '-T-'
          case ( 'Q' )
            filekey = 'Q'
            mf%paramkeys= '-Q-'
          case ( 'oro' )
            mf%paramkeys= '-oro-'
            mlevs = 'sfc'
            filekey = 'oro'
            year1  = 0
            month1 = 0
            day1   = 0
            treskey = '00p00'
            with_spm = .false.
            constant = .true.
          case ( 'spm' )
            mlevs = 'sfc'
            filekey = 'SP'
            mf%paramkeys= '-spm-'
            with_spm = .false.
          case default
            write (*,'("ERROR - unsupported grib paramkey `",a,"`")') paramkey
            write (*,'("ERROR in ",a)') name; status=1; return
        end select
        ! * create filenames
        write (mf%filename,'(a,"/",a,"-",a,"-",i4.4,"-",i2.2,"-",a,"-",a,"-",a,"_",i4.4,i2.2,i2.2,"_",a,".gb")') &
            trim(mf%dir_disk), &
            trim(mclass), trim(mtype), year1, month1, trim(mlevs), trim(mgrid), &
            trim(filekey), year1, month1, day1, trim(treskey)
        if ( with_spm ) then
          write (mf%spm_filename,'(a,"/",a,"-",a,"-",i4.4,"-",i2.2,"-",a,"-",a,"-",a,"_",i4.4,i2.2,i2.2,"_",a,".gb")') &
              trim(mf%dir_disk), &
              trim(mclass), trim(mtype), year1, month1, 'sfc', trim(mgrid), &
              'SP', year1, month1, day1, trim(treskey)
        else
          mf%spm_filename = 'none'
        endif

        ! open file(s) :
        select case ( mf%filetype )
#ifdef with_grib
          case ( 'grib' )
            !write (*,'("    open grib file ...")')
            call Init( mf%mf_grib, mf%filename, mf%spm_filename, constant, status )
            if (status/=0) then; write (*,'("ERROR in ",a)') name; status=1; return; end if
#endif
          case default
            write (*,'("ERROR - unsupported filetype `",a,"`")') trim(mf%filetype)
            write (*,'("ERROR -   sourcetype : ",a)') trim(sourcetype)
            write (*,'("ERROR in ",a)') name; status=1; return
        end select


      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! prism meteo in memory
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifdef with_prism  
      case ( 'prism' )
    
        ! only the requested parameter is provided by this prism 'file' ...
        mf%paramkeys = '-'//paramkey//'-'

        ! infinite time range ...
        mf%t1 = NewDate( year=1900, month=1, day=1, hour=1 )
        mf%t2 = NewDate( year=9999, month=9, day=9, hour=9 )
        !call PrintDate( '     fields valid from : ', mf%t1 )
        !call PrintDate( '                  to   : ', mf%t2 )

        ! set file type and file name:
        mf%filetype = 'prism'
        mf%filename = 'dummy'

        ! setup prism access;
        ! return list of available parameters
        !write (*,'("    setup prism access for ",a," ...")') paramkey
        call Init( mf%mf_prism, paramkey, status )
        if (status/=0) then; write (*,'("ERROR in ",a)') name; status=1; return; end if
        
#endif

      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ! error ...
      ! ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      case default

        write (*,'("ERROR - unsupported sourcetype `",a,"`")') trim(sourcetype)
        write (*,'("ERROR in ",a)') name; status=1; return

    end select

    ! file is opened (or, at least file name is known)
    mf%opened = .true.
    
    ! ok
    status = 0

  end subroutine mf_Setup
  
  

  ! ===========================================================
  !
  !   check contents of open meteo file
  !
  ! ===========================================================
  

  ! Check time in meteo file;
  ! status: 
  !   <0    : mf does not include [t1,t2]
  !    0    : mf includes [t,t2]
  !   >0    : error; mf not open ?
  !       
  
  subroutine mf_CheckTime( mf, t1, t2, status )

    use GO, only : TDate, operator(<), operator(<=), PrintDate
    
    ! --- begin -------------------------------------------
    
    type(TMeteoFile), intent(in)           ::  mf
    type(TDate), intent(in)                ::  t1, t2
    integer, intent(out)                   ::  status
    
    ! --- const -------------------------------------------
    
    character(len=*), parameter ::  name = mname//', mf_CheckTime'
    
    ! --- begin -------------------------------------------
    
    ! not open ?
    if ( .not. Opened(mf) ) then
      write (*,'("ERROR - meteo file not opened")')
      write (*,'("ERROR in ",a)') name; status = 1; return
    end if
    
    ! check [t1,t2]
    if ( t2 < t1 ) then
      write (*,'("ERROR - arguments should specify an interval :")')
      call PrintDate( 'ERROR -   t1 : ', t1 )
      call PrintDate( 'ERROR -   t2 : ', t2 )
      write (*,'("ERROR in ",a)') name; status = 1; return
    end if
    
    ! [t1,t2] is either:
    !   older than mf   ->  status = -2
    !   covered by mf   ->  status =  0
    !   newer than mf   ->  status = -1
    !   error ... (half in, half outside mf)
    if ( t2 < mf%t1 ) then
      status = -2; return
    else if ( (mf%t1 <= t1) .and. (t2 <= mf%t2) ) then
      status =  0; return
    else if ( mf%t2 <= t1 ) then
      status = -1; return
    else
      write (*,'("ERROR - requested interval [t1,t2] overlaps part of mf time:")')
      call PrintDate( 'ERROR -   t1    : ', t1 )
      call PrintDate( 'ERROR -   t2    : ', t2 )
      call PrintDate( 'ERROR -   mf%t1 : ', mf%t1 )
      call PrintDate( 'ERROR -   mf%t2 : ', mf%t2 )
      write (*,'("ERROR in ",a)') name; status = 1; return
    end if
    
  end subroutine mf_CheckTime
    
  
  ! ***
  
  
  ! Check if param is included in meteo file;
  ! status: 
  !   <0    : mf does not include param
  !    0    : mf includes param
  !   >0    : error; mf not open ?
  !       
  
  subroutine mf_CheckParam( mf, sourcekey, paramkey, status )

    ! --- begin -------------------------------------------
    
    type(TMeteoFile), intent(in)           ::  mf
    character(len=*), intent(in)           ::  sourcekey
    character(len=*), intent(in)           ::  paramkey
    integer, intent(out)                   ::  status
    
    ! --- const -------------------------------------------
    
    character(len=*), parameter ::  name = mname//', mf_CheckParam'
    
    ! --- local --------------------------------------------
    
    integer             ::  pos
    
    ! --- begin -------------------------------------------
    
    ! not open ?
    if ( .not. Opened(mf) ) then
      !write (*,'("ERROR - meteo file not opened")')
      !write (*,'("ERROR in ",a)') name; 
      status = 1; return
    end if
    
    ! by default not found ..
    status = -1
    
    ! wrong grid ? then leave
    if ( sourcekey /= mf%sourcekey ) return
    
    ! param list is for example: '-ps-pu-pv-',
    ! thus search for example for '-pu-' ...
    pos = index( mf%paramkeys, '-'//trim(paramkey)//'-' )
    if ( pos < 1 ) return
    
    ! ok
    status = 0

  end subroutine mf_CheckParam
    
    
    
  ! ===========================================================
  !
  !   read fields, grid definition, etc
  !
  ! ===========================================================
  

  subroutine mf_ReadRecord( mf, paramkey, t1, t2, nuv, gridtype, levi, &
                                lli, ll, sp_ll, &
                                ggi, gg, sp_gg, &
                                shi, sh, lnsp_sh, &
                                status )

    use GO, only : TDate, operator(+), operator(-), operator(/)
    use Grid, only : TllGridInfo, TggGridInfo, TshGridInfo, TLevelInfo

#ifdef with_hdf  
    use tmm_mf_hdf  , only : ReadRecord
#endif
#ifdef with_grib  
    use tmm_mf_grib , only : ReadRecord
#endif
#ifdef with_prism  
    use tmm_mf_prism, only : ReadRecord
#endif

    ! --- in/out -------------------------------
    
    type(TMeteoFile), intent(inout)      ::  mf
    character(len=*), intent(in)         ::  paramkey
    type(TDate), intent(in)              ::  t1, t2
    character(len=1), intent(in)         ::  nuv
    character(len=2), intent(out)        ::  gridtype
    type(TLevelInfo), intent(out)        ::  levi
    type(TllGridInfo), intent(inout)     ::  lli
    real, pointer                        ::  ll(:,:,:)
    real, pointer                        ::  sp_ll(:,:)
    type(TggGridInfo), intent(inout)     ::  ggi
    real, pointer                        ::  gg(:,:)
    real, pointer                        ::  sp_gg(:)
    type(TshGridInfo), intent(inout)     ::  shi
    complex, pointer                     ::  sh(:,:)
    complex, pointer                     ::  lnsp_sh(:)
    integer, intent(out)                 ::  status
    
    ! --- const --------------------------------------
    
    character(len=*), parameter  ::  name = mname//', mf_ReadRecord'
    
    ! --- local --------------------------------------
    
    type(TDate)         ::  tmid
    
    ! --- begin ---------------------------------
    
    !write (*,'(a,": begin")') name

    select case ( mf%filetype )
#ifdef with_hdf  
      case ( 'hdf' )
        call ReadRecord( mf%mf_hdf, paramkey, t1, t2, nuv, gridtype, levi, &
                         lli, ll, sp_ll, status, weaktest=mf%qad )
        if (status/=0) then; write (*,'("ERROR in ",a)') name; status=1; return; end if
#endif
#ifdef with_grib  
      case ( 'grib' )
        tmid = t1 + (t2-t1)/2
        call ReadRecord( mf%mf_grib, paramkey, tmid, tmid, nuv, gridtype, levi, &
                         lli, ll, sp_ll, status )
        if (status/=0) then; write (*,'("ERROR in ",a)') name; status=1; return; end if
#endif
#ifdef with_prism  
      case ( 'prism' )
        !call ReadRecord( mf%mf_prism, paramkey, t1, t2, nuv, gridtype, levi, &
        !                 lli, ll, sp_ll, ggi, gg, sp_gg, shi, sh, lnsp_sh, &
        !                 status )
        call ReadRecord( mf%mf_prism, paramkey, t1, t2, nuv, gridtype, levi, &
                         shi, sh, lnsp_sh, &
                         status )
        if (status/=0) then; write (*,'("ERROR in ",a)') name; status=1; return; end if
#endif
      case default
        write (*,'("ERROR - unsupported filetype `",a,"`")') trim(mf%filetype)
        write (*,'("ERROR in ",a)') name; status=1; return
    end select
    
    ! ok
    status = 0

    !write (*,'(a,": end")') name

  end subroutine mf_ReadRecord


  ! ***


  subroutine mf_ReadEqvLatStuff( mf, t1, t2, eqvlatb, eqvinds, status )
  
    use GO, only : TDate

#ifdef with_hdf  
    use tmm_mf_hdf, only : ReadEqvLatStuff
#endif

    ! --- in/out -------------------------------
    
    type(TMeteoFile), intent(inout)      ::  mf
    type(TDate), intent(in)              ::  t1, t2
    real, intent(out)                    ::  eqvlatb(:,:)
    integer, intent(out)                 ::  eqvinds(:,:,:)
    integer, intent(out)                 ::  status
    
    ! --- const --------------------------------------
    
    character(len=*), parameter  ::  name = mname//', mf_ReadEqvLatStuff'
    
    ! --- begin ---------------------------------

    select case ( mf%filetype )
#ifdef with_hdf  
      case ( 'hdf' )
        call ReadEqvLatStuff( mf%mf_hdf, t1, t2, eqvlatb, eqvinds, status )
        if (status/=0) then; write (*,'("ERROR in ",a)') name; status=1; return; end if
#endif
      case default
        write (*,'("ERROR - unsupported filetype `",a,"`")') trim(mf%filetype)
        write (*,'("ERROR in ",a)') name; status=1; return
    end select
    
    ! ok
    status = 0

  end subroutine mf_ReadEqvLatStuff
  

end module tmm_mf
